//****************************************************************************//
//   P1.cpp - ffnet eine eigene Client-Area und gibt dort einen Text aus     //
//****************************************************************************//
// Include-Datei <windows.h> ist unverzichtbar, sie enthlt alle wichtigen    //
// defines, die fr ein Windows-Programm bentigt werden                      //
//----------------------------------------------------------------------------//
#include <windows.h>

//----------------------------------------------------------------------------//
// Prototyp der Handlerfunction, die die Messages der Client-Area verarbeitet //
// Das Schlsselwort PASCAL ist notwendig, weil das Windowskernel ursprng-   //
// lich in Pascal programmiert wurde. Pascal und C legen die Parameter in     //
// unterschiedlicher Reihenfolge auf dem Stack ab, so da der C-Funktion mit- //
// geteilt werden mu, da die Parameter in anderer Reihenfolge vom Stack ab- //
// zuholen sind                                                               //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);
BOOL MessagePreprocess (HWND hwin, MSG &aMessage);

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main" - aber wann  //
// beginnt Micorsoft sich an bereits bestehende Konventionen zu halten ?      //
//----------------------------------------------------------------------------//
// Das pragma "argsused" ist Borland-spezifisch und unterdrckt das Warning,  //
// da wir den Standardparameter "lpszCmdParam" nirgendwo benutzt haben...    //
//----------------------------------------------------------------------------//
// in Parameter "hInstance"                                                   //
// bergibt das Windowskernel die Kennung des Programms. Dies ist ntig, da   //
// unter Windows ja mehrere Programme Parallel laufen knnen. Jedes Programm  //
// erhlt deshalb eine eindeutige ID-Nummer, ber welche die "Messages" zuge- //
// stellt werden knnen.                                                      //
//----------------------------------------------------------------------------//
// 16-Bit Windows: Im Parameter "hPrevInstance" teilt Windows mit, ob das     //
// gleiche Programm bereits im Speicher vorliegt, also erneut gestartet       //
// worden ist. In diesem Fall drfen einige Teile des Programmcodes nicht     //
// erneut durchlaufen werden. "hPrevInstance" ist NULL, wenn das Programm zum //
// ersten Mal in den Speicher geladen wird, ansonsten erhlt man hier die     //
// Instanzen-Nummer des vorherigen Aufrufes.                                  //
// 32-Bit Windows: Parameter ist immer NULL                                   //
//----------------------------------------------------------------------------//
// in Parameter "lpszCmdParam"                                                //
// stehen die Kommandozeilen-Parameter, die ggf. beim Aufruf bergeben wurden //
//----------------------------------------------------------------------------//
// in Parameter "nCmdShow"                                                    //
// ist festgelegt, ob das Programm als Pictogramm auf der Oberflche erscheint//
// (inconified) oder als Fenster.                                             //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdParam,
                    int nCmdShow)
{
   //-------------------------------------------------------------------------//
   // Windows bentigt einen eindeutigen Namen (die Fensterklasse), um fest-  //
   // stellen zu knnen, ob das gleiche Programm schon luft. Ein Kopieren    //
   // und Umbenennen der Exe-Datei gengt also nicht, um Windows zu ber-     //
   // listen... Der Sinn dahinter ist, da bei einem erneuten Aufruf des Pro- //
   // gramms nur der Datenbereich und nicht der Programmbereich neu angelegt  //
   // wird.                                                                   //
   // PRFIX "sz" steht fr String (String Zeroterminated)                    //
   //-------------------------------------------------------------------------//
	static char szWindowclassname [] = "P1";

   //-------------------------------------------------------------------------//
	// Datentyp HWND bedeutet Window-Handle, also eindeutige Kennnummer des    //
   // Fensters                                                                //
   // PRFIX "h" steht fr Handle, also eine eindeutige Kennziffer            //
   //-------------------------------------------------------------------------//
   HWND        hWindow;

   //-------------------------------------------------------------------------//
   // Datentyp MSG bedeutet Message, eine Struct, die die Daten einer an das  //
   // Programm weitergeleiteten Nachricht enthlt                             //
   // PRFIX "a" steht fr eine Struct                                        //
   //-------------------------------------------------------------------------//
   MSG         aMessage;

   //-------------------------------------------------------------------------//
   // Datentyp WNDCLASS meldet die Client-Area des Programms beim Windows-    //
   // Message-Dispatcher als Nachrichtenempfnger an                          //
   // PRFIX "a" steht fr eine Struct                                        //
   //-------------------------------------------------------------------------//
   WNDCLASS    aWindowclass;

   //-------------------------------------------------------------------------//
	// Die Abfrage auf hPrevInstance stellt sicher, da der Ablaufcode eines   //
   // Programms immer nur einmal geladen ist. Bei Multi-Instanz-Programmen    //
   // wird immer nur der Datenteil neu angelegt (16Bit)                       //
   // Unter 32-Bit-Windows ist dieser Parameter immer NULL! - sollte aber bei //
   // Multi-Plattformprogrammen (Programme fr Win3.x und Win95) sicherheits- //
   // halber abgefragt werden                                                 //
   //-------------------------------------------------------------------------//
   if (!hPrevInstance)
   {
   	//----------------------------------------------------------------------//
		// Wenn das Programm als erste Instanz in den Speicher geladen wird, mu//
      // da Hauptfenster (die Client-Area) beim Windows-Kernel angemeldet    //
      // werden. Dazu ist eine WNDCLASS-Sturct auszufllen, die das Fenster   //
      // als Fensterklasse anmeldet und ein bestimmtes Aussehen vereinbart    //
   	//----------------------------------------------------------------------//
		// Der Class-Style (CS_xxxx) legt bestimmte Verhaltensweisen des Haupt- //
      // fensters (d.h. der Client-Area) fest. Am hufigsten werden verwendet://
		// CS_HREDRAW = Gibt an, da das gesamte Fenster neu zu zeichnen ist,   //
      //              wenn Das Fenster in horizontaler Richtung bewegt oder   //
      //              in seiner Breite verndert wurde                        //
		// CS_VREDRAW = Gibt an, da das gesamte Fenster neu zu zeichnen ist,   //
      //              wenn Das Fenster in vertikaler Richtung bewegt oder     //
      //              in seiner Hhe verndert wurde                          //
		// CS_NOCLOSE = Schaltet das "Schliesymbol" in der obersten Zeile ab   //
		// CS_DBLCLKS = Weist Windows an, eine Doppelclick-Message an die       //
      //              Message-Handler-Function des Programms zu schicken, wenn//
      //              der Benutzer einen Doppelclick in irgendeinem der zur   //
      //              Anwendung gehrenden Fenster eingibt.                   //
      // Auerdem gibt es noch die seltener verwendeten Styles:               //
      // CS_BYTEALIGNCLIENT, CS_BYTEALIGNWINDOW, CS_CLASSDC,CS_GLOBALCLASS,   //
		// CS_OWNDC, CS_PARENTDC und CS_SAVEBITS                                //
   	//----------------------------------------------------------------------//
   	aWindowclass.style         = CS_HREDRAW | CS_VREDRAW;

   	//----------------------------------------------------------------------//
		// Name der Function, die in diesem Programm fr die Abarbeitung der    //
      // Messages sorgt. Die Funktion "DispatchMessage" ruft diesen Message-  //
      // Handler automatisch auf, weil er Teil der Klassenbeschreibung ist    //
      // entspricht einem Pointer auf diese Function                          //
   	//----------------------------------------------------------------------//
      aWindowclass.lpfnWndProc   = WndProc;

   	//----------------------------------------------------------------------//
		// Speicherplatz, der fr eigene Daten im Rahmen der Fensterklasse und  //
      // des Fensters vom Benutzer als Erweiterung der Struktur vorgesehene   //
      // sind. Wird hier nicht und auch sonst selten benutzt                  //
   	//----------------------------------------------------------------------//
      aWindowclass.cbClsExtra    = 0;
      aWindowclass.cbWndExtra    = 0;

   	//----------------------------------------------------------------------//
		// Die Erst-Instance wird ebenfalls in der Struktur gesichert, bei      //
      // weiteren Instancen sorgt Windows fr die Fortfhrung der Zhlerei in //
      // einer Liste (schlielich mu ja auch die hPrevInstance irgendwo her- //
      // kommen, d.h. Windows mu sie verwalten                               //
   	//----------------------------------------------------------------------//
      aWindowclass.hInstance     = hInstance;

      //----------------------------------------------------------------------//
      // Angabe des Icons, welches angezeigt wird, wenn das Programm als Icon //
      // auf den Desktop heruntergeclickt wird. Hier das Standardicon         //
      //----------------------------------------------------------------------//
      aWindowclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);

      //----------------------------------------------------------------------//
      // Angabe des Mouse-Cursors, der angezeigt wird, wenn ddie Mouse sich   //
      // im Progframmfenster befindet. Hier der Standardcursor                //
      //----------------------------------------------------------------------//
      aWindowclass.hCursor       = LoadCursor (NULL, IDC_ARROW);

      //----------------------------------------------------------------------//
      // Angabe des Hintergrund-Pinsels (Farbe/Muster), mit dem die Flche des//
      // Fensters bei Programmstart gefllt wird. Hier ein Standardbrush      //
      //----------------------------------------------------------------------//
      aWindowclass.hbrBackground = GetStockObject (WHITE_BRUSH);

      //----------------------------------------------------------------------//
      // Angabe der Menstruktur, die benutzt werden soll um das Men des     //
      // Fensters zu bilden. Hier Null, weil kein Men verwendet wird.        //
      //----------------------------------------------------------------------//
      aWindowclass.lpszMenuName  = NULL;

      //----------------------------------------------------------------------//
		// Eintragen des Klassennamens (s.o.) zur eindeutigen Identifikation    //
      // Meistens wird der Name des Programms verwendet                       //
      //----------------------------------------------------------------------//
      aWindowclass.lpszClassName = szWindowclassname;

      //----------------------------------------------------------------------//
		// Nachdem das "Formular" zur Beantragung einer Klasseneintragung im    //
      // Windowskernel korrekt ausgefllt ist, mu es beim Windowskernel auch //
      // eingereicht werden                                                   //
      //----------------------------------------------------------------------//
      if (!RegisterClass (&aWindowclass))
      {
         MessageBox (NULL, "Registrierung nicht mglich", "P1", MB_OK);
         return (0);
      }
   }

   //-------------------------------------------------------------------------//
	// Nachdem wir die Klasse registriert haben, sind wir beim Kernel (dem     //
   // Einwohnermeldeamt von Windows) bekannt. Um nun ein Fenster erffnen zu  //
   // drfen rufen wir die Function CreateWindow auf. Erst  dadurch wird      //
   // intern der Speicherplatz geschaffen, um ein Fenster zu verwalten. Dieses//
   // erste Fenster ist zugleich das wichtigste, denn es ist Bezugspunkt fr  //
   // alle weiteren Unterfenster. Das Hauptfenster wird daher auch Kunden-    //
   // bereich (Client-Area) genannt. Zudem knnen wir in groben Zgen fest-   //
   // legen, wie das Fenster aussehen soll.                                   //
   // Als Ergebnis der Funktion erhalten wir ein Handle auf das erzeugte      //
   // Fenster, also eine eindeutige Identifikationsnummer, mit der das Fen-   //
   // ster bestimmt werden kann.                                              //
   //-------------------------------------------------------------------------//
   // Parameter 1: Name der Fensterklasse, zu dem die Client-Area gehrt      //
   //-------------------------------------------------------------------------//
	// Parameter 2: Text, der in der Titelzeile des Fensters steht.            //
   //-------------------------------------------------------------------------//
	// Parameter 3: Fensterstil. Mgliche Werte sind (mit dem binr-Oder-      //
   //              Operator verbunden:                                        //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
   // Titelzeilen-Elemente                                                    //
	// WS_CAPTION          Fenster mit Titelzeile (beinhaltet WS_BORDER Stil)  //
	// WS_MAXIMIZEBOX	     Fenster hat Vergrerungsbutton in Titelzeile       //
	// WS_MINIMIZEBOX	     Fenster hat Verkleinerungsbutton in der Titelzeile  //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
	// Randelemente                                                            //
	// WS_BORDER	        Fenster mit schmalem Rand                           //
	// WS_THICKFRAME       Fenster mit breitem Rand, ber den Fenstergre     //
   //                     eingestellt werden kann. Identisch mit WS_SIZEBOX   //
   // WS_SIZEBOX    	     siehe WS_THICKFRAME                                 //
	// WS_HSCROLL          Erzeugt ein Fenster mit einer senkrechten Scrollbar //
	// WS_VSCROLL          Erzeugt ein Fenster mit einer horizontal-Scrollbar  //
	//	WS_SYSMENU          Erzeugt ein Fenster mit dem System-Men in der      //
   //                     Titelzeile. Dazu mu Stil WS_CAPTION gesetzt sein   //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
	// Zusammengefate Elemente                                                //
   // WS_OVERLAPPED       Erzeugt sogenanntes OVERLAPPED-Fenster. Hat immer   //
   //                     Titelzeile (WS_CAPTION) und Rand (WS_BORDER). Iden- //
   //                     tisch mit WS_TILED                                  //
	// WS_TILED            siehe WS_OVERLAPPED                                 //
	// WS_DLGFRAME         Fenster mit dem typischen Aussehen einer Dialogbox  //
   //                     Kann keine Titelzeile (WS_CAPTION) haben!           //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
	// Darstellungsgren                                                      //
	//	WS_ICONIC           Erzeugt Fenster, das zu Beginn als Icon vorliegt.   //
	// WS_MAXIMIZE         Erzeugt ein Fenster maximaler Gre.                //
	// WS_MINIMIZE         identisch mit WS_ICONIC                             //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
	// sonstige Eigenschaften                                                  //
	// WS_DISABLED         Fenster ist abgeschaltet                            //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
	// Zusammenfassende Stil-Pakete                                            //
	// WS_OVERLAPPEDWINDOW Erzeugt ein Fenster mit den Stilen WS_OVERLAPPED,   //
   //                     WS_CAPTION, WS_SYSMENU, WS_THICKFRAME,              //
   //                     WS_MINIMIZEBOX, und WS_MAXIMIZEBOX.                 //
	// WS_TILEDWINDOW      identisch mit WS_OVERLAPPEDWINDOW                   //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
	// Stile fr abhngige Fenster (Popupfenster / Childfenster)               //
	// WS_CHILD            Definiert ein Unterfenster (nicht bei Client-Area   //
   //                     mglich!) Schliet sich mit WS_POPUP gegenseitig aus//
	// WS_CHILDWINDOW	     identisch mit WS_CHILD                              //
	// WS_CLIPCHILDREN     Die Bereiche von Child-Fenstern werden ausgelassen, //
   //                     wenn das bergeordnete Fenster neu gezeichnet wird. //
	// WS_CLIPSIBLINGS     Die Bereiche von berlappenden Child-Fenstern       //
   //                     werden beachtet, so da nicht in die berlappenden  //
   //                     Bereiche geschrieben/gezeichnet wird                //
	// WS_POPUP            Erzeugt ein Dialogfenster. (nicht fr Client-Area   //
   //                     mglich!) Schliet sich mit WS_CHILD gegenseitig aus//
	// WS_POPUPWINDOW      Erzeugt ein Dialogfenster mit Standardaussehen.     //
   //                     Beinhaltet WS_BORDER, WS_POPUP, und WS_SYSMENU.     //
   //                     Mu mit WS_CAPTION kombiniert werden, um das        //
   //                     System-Men sichtbar zu machen                      //
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
	// Stile fr Dialogelemente                                                //
	// WS_GROUP            Bezeichnet das erste Fenster einer Child-Gruppe.    //
   //                     Wird fr die Fenster von Buttons und andere Dialog- //
   //                     elemente verwendet. Zwischen Fenstern der Gruppe    //
   //                     kann der Focus mit den Cursortasten bewegt werden.  //
   //                     Das nchste Fenster mit dem WS_GROUP beendet die    //
   //                     Gruppe wieder und startet eine neue                 //
	// WS_TABSTOP          Besagt, da Dialogelement mittels Tabulatortaste    //
   //                     erreicht werden kann.                               //
	// WS_VISIBLE          Erzeugt sichtbares Dialogelement. Unsichtbare       //
   //                     Elemente sind zugleich auf WS_DISABLED geschaltet   //
   //-------------------------------------------------------------------------//
   // Parameter 4: Fensterposition X-Koordinate. Angabe in Pixel. Die Angabe  //
   //              von CW_USEDEFAULT berlt die Entscheidung dem  Kernel    //
   //-------------------------------------------------------------------------//
   // Parameter 5: Fensterposition Y-Koordinate. Angabe in Pixel. Die Angabe  //
   //              von CW_USEDEFAULT berlt die Entscheidung dem  Kernel    //
   //-------------------------------------------------------------------------//
   // Parameter 6: Fensterbreite in Pixeln. Die Angabe von CW_USEDEFAULT      //
   //              berlt die Entscheidung dem Kernel                       //
   //-------------------------------------------------------------------------//
   // Parameter 7: Fensterhhe in Pixeln. Die Angabe von CW_USEDEFAULT ber-  //
   //              lt die Entscheidung dem Kernel                           //
   //-------------------------------------------------------------------------//
	// Parameter 8: Handle des bergeordneten Fensters. Da es sich hier um die //
   //              Client-Area handelt, gibt es kein bergeordnetes Fenster   //
   //-------------------------------------------------------------------------//
	// Parameter 9: Handle des Mens, welches im Fenster dargestellt werden    //
   //              soll. Da hier kein Men vereinbart wurde ist der Wert      //
   // 				 logischerweise NULL. Bei CHILD-Fenstern wird dieser Ein-   //
   //              trag als eindeutige ID-Nummer benutzt                      //
   //-------------------------------------------------------------------------//
	// Parameter 10: Instanzenhandle. Hier wird eingetragen, zu welcher Auf-   //
   //               rufinstanz das Fenster gehrt. Das ist wichtig, damit in  //
   //               den Dialogen die richtigen Datenbereiche angezeigt werden //
   //-------------------------------------------------------------------------//
	// Parameter 11. Pointer fr die in der Klasse reservierten Speicher-      //
   //               bereiche (s.o.) wird nur selten genutzt.                  //
   //-------------------------------------------------------------------------//
   hWindow = CreateWindow (szWindowclassname, "Hurra, es geht !!!",
                           WS_OVERLAPPEDWINDOW,
							      CW_USEDEFAULT, CW_USEDEFAULT,
							      CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);
   if (!hWindow)
   {
      MessageBox (NULL, "Fenster konnte nicht erzeugt werden", "P1", MB_OK);
      return (0);
   }

   //-------------------------------------------------------------------------//
	// Das Erzeugen des Fensters beinhaltet nicht, da das Fenster auch ange-  //
   // zeigt wird. Gelegentlich will man ja etwas im Hintergrund tun und das   //
   // zugehrige Fenster nur bei Bedarf anzeigen. Deshalb mu das Aufblenden  //
   // des Fensters ausdrcklich aufgerufen werden.                            //
   // Als Parameter bergibt man das Handle des erzeugten Fensters (Identifi- //
   // kationsnummer) und den Darstellungsmodus, der als Eigenschaft (z.B.     //
   // "als Icon") von Windows als Parameter an WinMain bergeben wurde.       //
   // Dieser Befehl darf nur EINMAL, hier im WinMain verwendet werden)        //
   //-------------------------------------------------------------------------//
   ShowWindow (hWindow, nCmdShow);

   //-------------------------------------------------------------------------//
	// Das Aufblenden des Fensters zeigt im Grunde nur die Flche an. Aus Ge-  //
   // schwindigkeitsgrnden werden die Fensterinhalte nur bei Bedarf oder     //
   // Aufforderung gemalt. Die folgende Zeile malt das ganze Fenster neu.     //
   // (Es wird eine WM_PAINT-Message ausgelst - s.u.)                        //
   //-------------------------------------------------------------------------//
   if (!UpdateWindow (hWindow))
      {
         MessageBox (NULL, "WM_PAINT fehlgeschlagen", "P1", MB_OK);
         return (0);
      }

   //-------------------------------------------------------------------------//
	// Das Fenster ist bereit und gezeichnet, jetzt mssen wir auf alle an-    //
   // kommenden Botschaften reagieren. GetMessage gibt solange TRUE zurck,   //
   // bis eine WM_QUIT-Message anzeigt, da das Programm beendet wurde. Die   //
   // WM_QUIT-Message ist die letzte Message, die ein Programm erhlt.        //
   // Bei ungltigen Messages (an nicht mehr vorhandenes Window-Handle) er-   //
   // gibt GetMessage den Wert -1                                             //
   //-------------------------------------------------------------------------//
   while (GetMessage (&aMessage, NULL, 0, 0))
   {
	   //----------------------------------------------------------------------//
		// Filterfunktion fr Messages allgemeiner Bedeutung, die nicht an      //
      // jedem einzelnen Control abgegriffen werden sollen                    //
	   //----------------------------------------------------------------------//
   	MessagePreprocess (hWindow, aMessage);

	   //----------------------------------------------------------------------//
		// TranslateMessage bersetzt die Windows-Tastaturcodes (Virtual Keys)  //
      // in Characterdaten. Natrlich nur, wenn solche Daten auch in der      //
      // Message vorliegen. Die MUSS gemacht werden, da man sonst mit den Key-//
      // messages nichts anfangen kann.                                       //
	   //----------------------------------------------------------------------//
   	TranslateMessage (&aMessage);

	   //----------------------------------------------------------------------//
		// DispatchMessage gibt die Message-struct an den Handler weiter, der   //
      // in der Windowsklasse eingetragen wurde (s.o.). Erst dort wird die    //
      // Nachricht verarbeitet. Sind mehrere Fenster vorhanden, so entschei-  //
      // det der Dispatcher anhand des in der Message eingetragenen Handles,  //
      // an welche Handlerroutine die Message geht.                           //
	   //----------------------------------------------------------------------//
      DispatchMessage (&aMessage);
   }

   //-------------------------------------------------------------------------//
	// Zu guter letzt, bevor wir uns verabschieden teilen wir dem Windows-     //
   // kernel noch mit, da wir die WM_QUIT-Message verarbeitet haben, indem   //
   // wir den wparam der Message zurckgeben.                                 //
   //-------------------------------------------------------------------------//
   return (aMessage.wParam);
}

//----------------------------------------------------------------------------//
// Funktion zum Herausfiltern der F1-Taste.                                   //
// Anschlieend wird die allgemeine Windowshilfe aufgerufen.                  //
//----------------------------------------------------------------------------//
BOOL MessagePreprocess (HWND hwin, MSG &aMessage)
{
   //-------------------------------------------------------------------------//
	// Nur regieren wenn eine WM_KEYUP (Taste losgelassen) Message vorliegt    //
   //-------------------------------------------------------------------------//
	if (WM_KEYUP == aMessage.message)
   {
	   //----------------------------------------------------------------------//
      // nur dann reagieren, wenn die losgelassene Taste die F1 Taste war     //
   	//----------------------------------------------------------------------//
		if (VK_F1 == (int)aMessage.wParam)
      {
		   //-------------------------------------------------------------------//
			// Die allgemeine Windowshilfe aufrufen                              //
		   //-------------------------------------------------------------------//
      	WinHelp (hwin, "C:\\WINDOWS\\HELP\\WINHLP32.HLP", HELP_INDEX, 0);
         return (TRUE);
      }
   }
   return (FALSE);
}

//----------------------------------------------------------------------------//
// Handlerfunktion der Client-Area, wird von DispatchMessage aufgerufen.      //
// Parameter "hwnd" gibt an, an welches Fensterhandle sich die Botschaft      //
//           richtet.                                                         //
// Parameter "message" gibt die Messageart an                                 //
// Parameter "wParam" ist ein Datenfeld, das abhngig von der Messageart zu   //
//                    interpretieren ist.                                     //
// Parameter "lParam" ist ein Datenfeld, das abhngig von der Messageart zu   //
//                    interpretieren ist.                                     //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND hwnd, UINT message, UINT wParam,
											LONG lParam)
{
   //-------------------------------------------------------------------------//
	// Handle des Devicecontextes. Der DC kmmert sich um die Belange des anzu-//
   // sprechenden Gertes (hier Bildschirm), so da wir uns nicht selbst um   //
   // die Umrechnung von Farben etc. auf die Bildschirmauflsung und Farb-    //
   // tiefe kmmern mssen. Dadurch funktionieren alle Windowsprogramme meist //
   // problemlos mit fas jeglicher, windowsfhiger Hardware. Der DC greift    //
   // auf die entsprechenden Treiber zu, ohne da der Entwickler sich darum   //
   // im Normalfall Gedanken machen mu.                                      //
   //-------------------------------------------------------------------------//
	HDC         hdc ;

   //-------------------------------------------------------------------------//
	// Die Paintstruct wird von Windows bentigt, wenn ein Teil des Fensters   //
   // neu zu zeichnen ist.                                                    //
   //-------------------------------------------------------------------------//
   PAINTSTRUCT ps ;

   //-------------------------------------------------------------------------//
	// rect ist eine Struktur, die einen rechteckigen Bereich des Bildschirms  //
   // beschreibt. Sie enthlt nur die Koordinaten der linken, oberen und der  //
   // rechten unteren Ecke.                                                   //
   //-------------------------------------------------------------------------//
   RECT        rect ;

   //-------------------------------------------------------------------------//
	// Der switch-Block reagiert auf die Messagearten, die von Windows als     //
   // Nachricht geschickt werden. Hier nur zwei von SEHR VIELEN (ca. 240)     //
   // verschiedenen Messagearten, die es gibt. WM steht fr Windowsmessage,   //
   // wobei diese unter Umstnden in den Parametern wParam und lParam noch    //
   // weitere Untergruppen oder Informationen beinhalten                      //
   //-------------------------------------------------------------------------//
   switch (message)
   {
      //----------------------------------------------------------------------//
		// Die WM_PAINT-Message gibt an, was alles zu tun ist, wenn die Client- //
      // Area neu zu zeichnen ist.                                            //
      // Windows schickt eine WM_PAINT-Message, wenn z.B. die Fenstergre    //
      // verndert wurde oder man das Fenster verschoben hat.                 //
      //----------------------------------------------------------------------//
   	case WM_PAINT :
	        //-----------------------------------------------------------------//
			  // BeginPaint leitet das Zeichnen ein. whrend dieses Vorgangs     //
           // kann in Windows kaum etwas anderes gemacht werden, der Bild-    //
           // schirm wird kurzfristig exclusiv fr dieses Programm gebraucht  //
           // NIEMALS hier einen Breakpoint setzen, denn das fhrt in 99%     //
           // aller Flle zu einem Absturz!                                   //
 	        //-----------------------------------------------------------------//
           hdc = BeginPaint (hwnd, &ps);

	        //-----------------------------------------------------------------//
			  // Hier werden die gerade gltigen Koordinaten der Fenstergre    //
           // der Client-Area ermittelt. Da der Anwender die Fenstergre     //
           // nahezu beliebig ndern kann, ist dies notwendig um die gerade   //
           // eingestellte Gre zu ermitteln                                 //
           // Der Bereich der ZUrckgegeben wird, ist der innere Fenster-     //
           // bereich (Ohne Titelzeile und Rand!). Das Ergebnis wird in die   //
           // Rechteck-Struktur (rect)                                        //
	        //-----------------------------------------------------------------//
           GetClientRect (hwnd, &rect);

	        //-----------------------------------------------------------------//
			  // Drawtext schreibt einen Text in die Client-Area.                //
           // 1. Parameter ist der HDC, den Windows in BeginPaint ermittelt   //
           //              hat.                                               //
           // 2. Parameter ist der auszugebende Text, oder der Pointer auf    //
           //              den auszugebenden Text.                            //
           // 3. Parameter ist die auszugebende Textlnge. Wird hier -1 ange- //
           //              geben, so zhlt Windows selber. Vorsicht, ist die  //
           //              Angabe Grer als die Textlnge, so steht Unsinn   //
           //              auf dem Bildschirm                                 //
           // 4. Parameter ist der rechteckige Bildschirmbereich, in den der  //
           //              Text ausgegeben werden soll                        //
           // 5. Parameter gibt an, wie die Ausgabe erfolgen soll. Die Anga-  //
           //              knnen ber binres Oder verknpft werden.         //
           //              Mgliche Angaben sind:                             //
			  //              DT_BOTTOM          Am unteren Randbereich des Rec- //
           //                                 tangles ausgeben. Nur zusammen  //
           //											 mit DT_SINGLELINE mglich       //
           //              DT_CALCRECT        Berechnet des bentigten Platz  //
           //                                 bei Multiline-Ausgaben selbst-  //
           //                                 ttig.                          //
           //					DT_CENTER          Zentriert den Text in der hori- //
           //                                 zontalen Ebene                  //
           //					DT_EXPANDTABS      Erweitert Tabzeichen auf die    //
           //                                 entsprechende Lnge. Vorgabe    //
           //                                 ist die Breite von 8 Blanks     //
           //					DT_EXTERNALLEADING Berechnet typographische Unter- //
           //                                 und berlngen in den bentig-  //
           //                                 ten Platz mit ein               //
           //					DT_LEFT            Linksbndige Ausgabe im ange-   //
           //                                 gebenen Rectangle               //
			  //					DT_NOCLIP			 Ausgabe ohne Bercksichtigung   //
           //                                 von verdeckten Bereichen. Ist   //
           //                                 schneller, macht aber ggf.      //
           //                                 Probleme mit Childwindows       //
           //					DT_NOPREFIX        Normalerweise wird ein im Text  //
           //                                 befindlichen "&"-Zeichen als    //
           //                                 Unterstreichung fr das folgen- //
           //                                 de Zeichen gewertet und ein     //
           //                                 "&&" als Ausgabe des &-Zeichens //
           //                                 Diese Methode kann ber diesen  //
           //                                 Stil abgeschaltet werden        //
           //					DT_RIGHT           Rechtsbndige Ausgabe im ange-  //
           //                                 gebenen Rectangle               //
			  //					DT_SINGLELINE      Einzeilige Ausgabe Zeilenvor-   //
           //                                 schubzeichen (CRLF / "\n") wer- //
           //                                 den nicht beachtet              //
			  //					DT_TABSTOP			 Setzt einen Tabulatorstop       //
			  //					DT_TOP	          Am oberen Randbereich des Rec-  //
           //                                 tangles ausgeben. Nur zusammen  //
           //                                 mit DT_SINGLELINE mglich       //
			  //					DT_VCENTER         Zentriert den Text in der verti-//
           //                                 kalen Ebene. Nur zusammen mit   //
           //                                 DT_SINGLELINE mglich           //
			  //					DT_WORDBREAK       Gibt an, da nur ganze Worte    //
           //                                 umgebrochen werden sollen, bei  //
           //                                 mehrzeiliger Ausgabe.           //
           //-----------------------------------------------------------------//
           DrawText (hdc, "unglaublich, aber es geht!", -1, &rect,
                     DT_SINGLELINE | DT_CENTER | DT_VCENTER);

           //-----------------------------------------------------------------//
           // Bendet die geschtze Zeichnerei                                 //
			  //-----------------------------------------------------------------//
			  EndPaint (hwnd, &ps);

           //-----------------------------------------------------------------//
			  // Abarbeitung der WM_PAINT-Message an Windeowskernel besttigen   //
           //-----------------------------------------------------------------//
           return (0);

      //----------------------------------------------------------------------//
		// Die WM_DESTROY-Message wird empfangen, also der Wunsch des Benutzers //
      // das Programm zu ebeenden. ggf. ist dies nicht mglich, dann kann     //
      // die Beendigung hier unterbunden werden. Ist die Beendigung erlaubt,  //
      // So ist hier die letzte Mglichkeit Programmresourcen und Speicher-   //
      // platz freizugeben, die dynamisch erzeugt worden sind !               //
      //----------------------------------------------------------------------//
      case WM_DESTROY :

	        //-----------------------------------------------------------------//
           // Wenn nichts dagegen spricht, dann hier das Programmende durch-  //
           // fhren. PostQuitMessage schickt eine WM-QUIT-Message an unser   //
           // eigenes Programm, so da die Handler-Schleife in WinMain (die   //
           // GetMessage-Schleife) beendet wird                               //
           //-----------------------------------------------------------------//
           PostQuitMessage (0);

           //-----------------------------------------------------------------//
			  // Abarbeitung der WM_DESTROY-Message an Windeowskernel besttigen //
           //-----------------------------------------------------------------//
           return (0);
   }
   //-------------------------------------------------------------------------//
	// da uns alle anderen Messages nicht interessieren, mssen wir dem Kernel //
   // mitteilen, da es die Nachricht als erledigt betrachten kann. Dazu wer- //
   // den sie formal in einem Standard-Handler des Kernels weiterverarbeitet. //
   //-------------------------------------------------------------------------//
   return DefWindowProc (hwnd, message, wParam, lParam);
}