//****************************************************************************//
//   P2.cpp - Ermittelt einige wichtige Systemgrenzen                         //
//****************************************************************************//

#include <windows.h>
#include "p2.h"

inline int min (int a, int b) { if (a < b) return (a); else return (b); }
inline int max (int a, int b) { if (a > b) return (a); else return (b); }

//----------------------------------------------------------------------------//
// Prototyp der Handlerfunction, die die Messages der Client-Area verarbeitet //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG) ;

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"              //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
{
	static char szWindowclassname[] = "P2" ;
   HWND        hWindow;
   MSG         aMessage;
   WNDCLASS    aWndclass;

   if (!hPrevInstance)
   {
   	aWndclass.style         = CS_HREDRAW | CS_VREDRAW;
      aWndclass.lpfnWndProc   = WndProc;
      aWndclass.cbClsExtra    = 0;
      aWndclass.cbWndExtra    = 0;
      aWndclass.hInstance     = hInstance;
      aWndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
      aWndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
      aWndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
	   aWndclass.lpszMenuName  = NULL;
      aWndclass.lpszClassName = szWindowclassname;

      RegisterClass (&aWndclass);
	}

   hWindow = CreateWindow (szWindowclassname, "Wichtige Mae des Systems",
                           WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);

	ShowWindow (hWindow, nCmdShow);
   UpdateWindow (hWindow);

   while (GetMessage (&aMessage, NULL, 0, 0))
   {
	   TranslateMessage (&aMessage);
      DispatchMessage (&aMessage);
   }
   return (aMessage.wParam);
}

long FAR PASCAL _export WndProc (HWND hwnd, UINT message, UINT wParam,
                                 LONG lParam)
{
   //-------------------------------------------------------------------------//
	// die folgenden Variablen sind vom Typ STATIC INT, weil sie nur einmal,   //
   // Beginn des Programms ermittelt werden, wenn Windows die WM_CREATE-      //
   // Message sendet. Da die WndProc-Function stndig aufgerufen und wieder   //
   // verlassen wird, ist es gnstiger die Daten zu erhalten                  //
   //-------------------------------------------------------------------------//
	// cxChar      - durchschnittliche Zeichenbreite (Windows-Standardfont)    //
	// cxCaps      - durchschnittliche Zeichenbreite von Grobuchstaben        //
   // cyChar      - Hhe eines Zeichens                                       //
   // cyClient    - aktuelle Hhe der Client-Area                             //
   // nMaxWidth   - maximale Breite                                           //
   // nVscrollPos -                                                           //
   // nVscrollMax -                                                           //
   // nHscrollPos -                                                           //
   // nHScrollMax -                                                           //
   //-------------------------------------------------------------------------//
	static int   cxChar, cxCaps, cyChar, cxClient, cyClient, nMaxWidth,
                nVscrollPos, nVscrollMax, nHscrollPos, nHscrollMax;
   char         szBuffer[50];
   HDC          hdc;
   int          i, x, y, nPaintBeg, nPaintEnd, nVscrollInc, nHscrollInc;
   PAINTSTRUCT  ps;
   TEXTMETRIC   tm;

	switch (message)
   {
   	//----------------------------------------------------------------------//
      // Windows sendet eine WM-CREATE-Message, kurz bevor das Fenster ange-  //
      // zeigt wird. Dies ist also der richtige Ort, um Daten zu ermitteln,   //
      // die dort angezeigt werden sollen. Aber Vorsicht! - jedes Fenster     //
      // bekommt diese Message nur ein einziges Mal! - Der Bereich eignet     //
      // sich also nur zur INITIALISIERUNG!                                   //
   	//----------------------------------------------------------------------//
   	case WM_CREATE :
	        //-----------------------------------------------------------------//
           // Device-Context ermitteln                                        //
           //-----------------------------------------------------------------//
           hdc = GetDC (hwnd) ;

           //-----------------------------------------------------------------//
			  // Wichtige Anzeige-Daten des Eingestellten Schrifttyps holen      //
           // als Zwischenpuffer dient die TEXTMETRIC-Struktur                //
           //-----------------------------------------------------------------//
           GetTextMetrics (hdc, &tm) ;

           //-----------------------------------------------------------------//
			  // Die Standardbreite eines Kleinbuchstaben entspricht blicher-   //
           // weise der Breite des Zeichens 'x'. Dieser wird in der TEXTMETRIC//
           // Struktur festgehalten und hier auf die Static-Variable kopiert  //
           //-----------------------------------------------------------------//
           cxChar = tm.tmAveCharWidth ;

           //-----------------------------------------------------------------//
			  // Die TEXTMETRIC-Struktur enthlt auch Informationen darber, ob  //
           // der Standardfont eine Proportionalschrift ist, oder ein Font    //
           // gleichbleibender Breite ist.                                    //
           // Ist das Bit TMPF_FIXED_PITCH gesetzt, so handelt es sich um     //
           // eine Schrift gleichbleibender Breite. D.h. alle Zeichen sind    //
           // gleich breit. Bei Proportionalfonts sind die Grobuchstaben in  //
           // etwa 1,5 mal so breit wie ein einfacher Buchstabe               //
           //-----------------------------------------------------------------//
           cxCaps = (tm.tmPitchAndFamily & TMPF_FIXED_PITCH	? 3:2) * cxChar/2;

           //-----------------------------------------------------------------//
		     // Die Komponente "tmHeight" enthlt die Hhe des Zeichens inkl.   //
           // der Unterlngen. "tmExternalLeading" beschreibt der Platz, der  //
           // zwischen zwei Zeilen bentigt wird, damit die Buchstaben nicht  //
           // aneinander "kleben"                                             //
           //-----------------------------------------------------------------//
           cyChar = tm.tmHeight + tm.tmExternalLeading ;

           //-----------------------------------------------------------------//
			  // Nicht vergessen, den DC wieder freigeben - Denn der DC ist eine //
           // limitierte Resource (es kann nur einen pro Gert, hier also     //
           // der Bildschirm, geben                                           //
           //-----------------------------------------------------------------//
           ReleaseDC (hwnd, hdc) ;

           //-----------------------------------------------------------------//
           // Die Zeile besteht aus etwa 22 Grobuchstaben (1. Spalte) und    //
           // einem Text von ca. 40 Zeichen (2. und 3. Spalte), so da die    //
           // Gesamtbreite gesschtzt werden kann als:                        //
           //-----------------------------------------------------------------//
           nMaxWidth = 40 * cxChar + 22 * cxCaps ;

           //-----------------------------------------------------------------//
			  // Den Wert Null zurckgeben, als Zeichen, da keine weitere Bear- //
           // beitung notwendig ist.                                          //
           //-----------------------------------------------------------------//
           return (0);

   	//----------------------------------------------------------------------//
      // Windows sendet eine WM-SIZE-Message, jedesmal, wenn die Gre des    //
      // Fensters gendert wird. Im Parameter "lParam" sind die neuen         //
      // Fenstergren abgreifbar.                                            //
   	//----------------------------------------------------------------------//
      case WM_SIZE :
           //-----------------------------------------------------------------//
           // Die y-Ausdehnung (Hhe) ist im Highword von lParam zu finden    //
           //-----------------------------------------------------------------//
           cyClient = HIWORD (lParam) ;

           //-----------------------------------------------------------------//
           // Die x-Ausdehnung (Breite) ist im Lowword von lParam zu finden   //
           //-----------------------------------------------------------------//
           cxClient = LOWORD (lParam) ;

           //-----------------------------------------------------------------//
           // Hier wird ermittelt, um wieviele Zeilen mit der vertikalen      //
           // Scrollbar geblttert werden kann. Dazu bentigt man zunchst    //
           // die Anzahl der berhaupt anzuzeigenden Zeilen (hier ber das    //
           // Makro NUMLINES realisiert). Davon abzuziehen ist die Anzahl der //
           // Zeilen, die bereits zu sehen ist. Diese kann ermittelt werden,  //
           // indem man die Hhe des Fensters durch die Hhe einer Zeile      //
           // teilt (die +2 sind ein Ausgleich fr den oberen und unteren     //
           // Rand der Client-Area). Subtrahiert man die Werte voneinander,   //
           // so erhlt man die Anzahl der nicht-sichtbaren Zeilen, also die  //
           // maximale Anzahl der zeilen, um die man blttern darf. Ist der   //
           // Wert negativ, so knnen im Fenster mehr Zeilen dargestellt wer- //
           // den als hier bentigt. in diesem Fall ist "nVscrollMax" auf     //
           // Null zu setzen (hier realisiert durch die INLINE-Function "max" //
           //-----------------------------------------------------------------//
	        nVscrollMax = max (0, (int)(NUMLINES + 2 - cyClient / cyChar));

           //-----------------------------------------------------------------//
			  // Sicherstellen, da die Scrollposition nicht grer ist, als die //
           // maximal mgliche...                                             //
           //-----------------------------------------------------------------//
           nVscrollPos = min (nVscrollPos, nVscrollMax);

           //-----------------------------------------------------------------//
			  // Den Scrollbereich setzen. Dieser liegt (in Zeilen) zwischen 0   //
           // und nVscrollMax... Der letzte Parameter besagt, ob das Fenster  //
           // neu gezeichnet werden soll                                      //
           //-----------------------------------------------------------------//
           SetScrollRange (hwnd, SB_VERT, 0, nVscrollMax, FALSE);

           //-----------------------------------------------------------------//
			  // Die Scrollposition (den Zieh-Button relativ zum eben er-        //
           // mittelten Range setzen                                          //
           //-----------------------------------------------------------------//
           SetScrollPos   (hwnd, SB_VERT, nVscrollPos, TRUE);

           //-----------------------------------------------------------------//
           // Hier wird ermittelt, um wieviele Zeilen mit der horizontalen    //
           // Scrollbar geblttert werden kann.                               //
           //-----------------------------------------------------------------//
           nHscrollMax = max (0, 2 + (nMaxWidth - cxClient) / cxChar) ;

           //-----------------------------------------------------------------//
			  // Sicherstellen, da die Scrollposition nicht grer ist, als die //
           // maximal mgliche...                                             //
           //-----------------------------------------------------------------//
           nHscrollPos = min (nHscrollPos, nHscrollMax) ;

           //-----------------------------------------------------------------//
			  // Den Scrollbereich setzen. Dieser liegt (in Zeilen) zwischen 0   //
           // und nHscrollMax... Der letzte Parameter besagt, ob das Fenster  //
           // neu gezeichnet werden soll                                      //
           //-----------------------------------------------------------------//
           SetScrollRange (hwnd, SB_HORZ, 0, nHscrollMax, FALSE);

           //-----------------------------------------------------------------//
			  // Die Scrollposition (den Zieh-Button relativ zum eben er-        //
           // mittelten Range setzen                                          //
           //-----------------------------------------------------------------//
           SetScrollPos   (hwnd, SB_HORZ, nHscrollPos, TRUE);
           return (0);

   	//----------------------------------------------------------------------//
		// Windows sendet eine VScroll-Message, wenn die Vertikale Bildlauf-    //
      // leiste bettigt wird                                                 //
   	//----------------------------------------------------------------------//
      case WM_VSCROLL :
           //-----------------------------------------------------------------//
			  // Im WPARAM der VScroll-Message steht, in welcher Form die Bild-  //
           // laufleiste bettigt wird                                        //
           //-----------------------------------------------------------------//
           switch (LOWORD(wParam))
           {
              //--------------------------------------------------------------//
              // Bettigungsform: gehe an den Beginn der Liste...             //
              //--------------------------------------------------------------//
              case SB_TOP :
                   //---------------------------------------------------------//
                   // nVscrollInc erhlt die RELATIVE Anzahl an Zeilen, um    //
                   // welche die Position gendert werden soll                //
                   //---------------------------------------------------------//
                   nVscrollInc = -nVscrollPos;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe an das Ende der Liste...               //
              //--------------------------------------------------------------//
              case SB_BOTTOM :
                   nVscrollInc = nVscrollMax - nVscrollPos;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe eine Zeile hoch...                     //
              //--------------------------------------------------------------//
              case SB_LINEUP :
                   nVscrollInc = -1;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe eine Zeile runter...                   //
              //--------------------------------------------------------------//
              case SB_LINEDOWN :
                   nVscrollInc = 1;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe eine Seite hoch...                     //
              //--------------------------------------------------------------//
              case SB_PAGEUP :
                   nVscrollInc = min (-1, -cyClient / cyChar);
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe eine Seite runter...                   //
              //--------------------------------------------------------------//
              case SB_PAGEDOWN :
                   nVscrollInc = max (1, cyClient / cyChar);
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: Mit der Maus den Scrollbutton ziehen...     //
              //--------------------------------------------------------------//
              case SB_THUMBTRACK :
                   nVscrollInc = HIWORD (wParam) - nVscrollPos;
                   break;

              //--------------------------------------------------------------//
              // sonstige Bettigungsform: mache gar nix...                   //
              //--------------------------------------------------------------//
              default :
                   nVscrollInc = 0;
           }

           //-----------------------------------------------------------------//
			  // Sicherstellen, da nicht ber die Randgrenzen gegangen wird     //
           //-----------------------------------------------------------------//
           nVscrollInc = max (-nVscrollPos, min (nVscrollInc, nVscrollMax -
                              nVscrollPos));

           //-----------------------------------------------------------------//
			  // Wenn nVScrollInc einen gltigen Wert enthlt, dann den Bildaus- //
           // schnitt auf den entsprechenden Teil setzen                      //
           //-----------------------------------------------------------------//
           if (nVscrollInc)
           {
           	  //--------------------------------------------------------------//
              // Scrollposition um die oben ermittelte  Verschiebung ndern   //
           	  //--------------------------------------------------------------//
              nVscrollPos += nVscrollInc;

           	  //--------------------------------------------------------------//
              // Fenster verschieben (hier wieder Pixel...)                   //
           	  //--------------------------------------------------------------//
              ScrollWindow (hwnd, 0, -cyChar * nVscrollInc, NULL, NULL);

           	  //--------------------------------------------------------------//
			     // Die Scrollposition (den Zieh-Button relativ zum Range setzen //
           	  //--------------------------------------------------------------//
              SetScrollPos (hwnd, SB_VERT, nVscrollPos, TRUE) ;

           	  //--------------------------------------------------------------//
              // Windows mitteilen, da das Fenster neu gezeichnet werden mu //
              // Windows generiert dann eine neue WM_Paint-Message fr dieses //
              // Fenster                                                      //
           	  //--------------------------------------------------------------//
              UpdateWindow (hwnd) ;
           }
           return (0);

   	//----------------------------------------------------------------------//
		// Windows sendet eine VScroll-Message, wenn die Vertikale Bildlauf-    //
      // leiste bettigt wird                                                 //
   	//----------------------------------------------------------------------//
      case WM_HSCROLL :
           //-----------------------------------------------------------------//
			  // Im WPARAM der HScroll-Message steht, in welcher Form die Bild-  //
           // laufleiste bettigt wird                                        //
           //-----------------------------------------------------------------//
           switch (LOWORD(wParam))
           {
              //--------------------------------------------------------------//
              // Bettigungsform: gehe einen Buchstaben nach links...         //
              //--------------------------------------------------------------//
              case SB_LINEUP :
                   nHscrollInc = -1;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe einen Buchstaben nach rechts...        //
              //--------------------------------------------------------------//
              case SB_LINEDOWN :
                   nHscrollInc = 1;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe acht Buchstaben nach links...          //
              //--------------------------------------------------------------//
              case SB_PAGEUP :
                   nHscrollInc = -8;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: gehe acht Buchstaben nach rechts...         //
              //--------------------------------------------------------------//
              case SB_PAGEDOWN :
                   nHscrollInc = 8;
                   break;

              //--------------------------------------------------------------//
              // Bettigungsform: Mit der Maus den Scrollbutton ziehen...     //
              //--------------------------------------------------------------//
              case SB_THUMBTRACK :
                   nHscrollInc = HIWORD (wParam) - nHscrollPos;
                   break;

              //--------------------------------------------------------------//
              // sonstige Bettigungsform: mache gar nix...                   //
              //--------------------------------------------------------------//
              default :
                   nHscrollInc = 0;
           }

           //-----------------------------------------------------------------//
			  // Sicherstellen, da nicht ber die Randgrenzen gegangen wird     //
           //-----------------------------------------------------------------//
           nHscrollInc = max (-nHscrollPos, min (nHscrollInc, nHscrollMax -
                              nHscrollPos));

           //-----------------------------------------------------------------//
			  // Wenn nHScrollInc einen gltigen Wert enthlt, dann den Bildaus- //
           // schnitt auf den entsprechenden Teil setzen                      //
           //-----------------------------------------------------------------//
           if (nHscrollInc)
           {
           	  //--------------------------------------------------------------//
              // Scrollposition um die oben ermittelte  Verschiebung ndern   //
           	  //--------------------------------------------------------------//
           	  nHscrollPos += nHscrollInc;

           	  //--------------------------------------------------------------//
              // Fenster verschieben (hier wieder Pixel...)                   //
           	  //--------------------------------------------------------------//
              ScrollWindow (hwnd, -cxChar * nHscrollInc, 0, NULL, NULL);

           	  //--------------------------------------------------------------//
			     // Die Scrollposition (den Zieh-Button relativ zum Range setzen //
           	  //--------------------------------------------------------------//
              SetScrollPos (hwnd, SB_HORZ, nHscrollPos, TRUE);

           	  //--------------------------------------------------------------//
				  // Ein Update Window ist hier nicht ntig, da der sichtbare Be- //
              // reich vom Windows-Clipping bestimmt wird.                    //
           	  //--------------------------------------------------------------//
           }
           return (0);

      case WM_PAINT :
           hdc = BeginPaint (hwnd, &ps) ;

           //-----------------------------------------------------------------//
           // Aufbau der Paint-Struktur:                                      //
			  // hdc         Der device-context des Displays, fr welches ge-    //
           //             zeichnet wird                                       //
           // fErase      Wenn dieser Wert ungleich Null ist, dann soll       //
           //             soll Windows beim Zeichnen den Hintergrund lschen  //
           // rcPaint	  Hierbei handelt es sich um eine RECTANGLE-Struktur, //
           //             die den zu zeichnenden Bereich beschreibt (linke    //
           //             obere und rechte untere Ecke                        //
           // Die restlichen Parameter sind reserviert und drfen nur vom     //
           // Windowssystem selbst benutzt werden                             //
           //-----------------------------------------------------------------//
           // nPaintBeg und nPaintEnd legen fest, wieviele Zeilen im momentan //
           // sichtbaren Fensterbereich berhaupt angezeigt werden knnen.    //
           // nPaintBeg legt fest, mit welcher Zeile gestartet werden soll,   //
           // nPaintEnd enthlt die Zeilennummer, mit der zu beenden ist.     //
           //-----------------------------------------------------------------//
           nPaintBeg = max (0, nVscrollPos + ps.rcPaint.top / cyChar - 1) ;
           nPaintEnd = min (NUMLINES, nVscrollPos+ps.rcPaint.bottom/cyChar);

           //-----------------------------------------------------------------//
           // Schleife von Zeilen nPaintBeg bis nPaintEnd                     //
           //-----------------------------------------------------------------//
           for (i = nPaintBeg ; i < nPaintEnd ; i++)
           {
				  //--------------------------------------------------------------//
              // Leider sind jetzt die Zeilennummern wieder in Pixel umzu-    //
              // rechnen... gleiches gilt natrlich auch fr die horizontale  //
              // Ausrichtung                                                  //
				  //--------------------------------------------------------------//
              x = cxChar * (1 - nHscrollPos);
              y = cyChar * (1 - nVscrollPos + i);

				  //--------------------------------------------------------------//
              // und jetzt die Textausgabe 1. Spalte                          //
				  //--------------------------------------------------------------//
              TextOut (hdc, x, y, sysmetrics[i].szLabel,
                      lstrlen (sysmetrics[i].szLabel)) ;

				  //--------------------------------------------------------------//
              // und jetzt die Textausgabe 2. Spalte                          //
				  //--------------------------------------------------------------//
              TextOut (hdc, x + 22 * cxCaps, y, sysmetrics[i].szDesc,
                       lstrlen (sysmetrics[i].szDesc));

				  //--------------------------------------------------------------//
              // und jetzt die Textausgabe 3. Spalte                          //
				  //--------------------------------------------------------------//
              SetTextAlign (hdc, TA_RIGHT | TA_TOP);
              TextOut (hdc, x + 22 * cxCaps + 40 * cxChar, y, szBuffer,
                       wsprintf (szBuffer, "%5d",
                                GetSystemMetrics (sysmetrics[i].nIndex)));
              SetTextAlign (hdc, TA_LEFT | TA_TOP);
           }
           EndPaint (hwnd, &ps);
           return (0);
      case WM_DESTROY :
           PostQuitMessage (0);
           return (0);
   }
   return DefWindowProc (hwnd, message, wParam, lParam);
}
