//****************************************************************************//
//   P6b.cpp - Timer-Demo 1                                                   //
//****************************************************************************//
#include <windows.h>

#define ID_TIMER    1

//----------------------------------------------------------------------------//
// Prototyp der Handlerfunction, die die Messages der Client-Area verarbeitet //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"              //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
{
	static char szWindowclassname[] = "P6b" ;
   HWND        hWindow;
   MSG         aMessage;
   WNDCLASS    aWndclass;
   int         nAntwort;

   if (!hPrevInstance)
   {
   	aWndclass.style         = CS_HREDRAW | CS_VREDRAW;
      aWndclass.lpfnWndProc   = WndProc;
      aWndclass.cbClsExtra    = 0;
      aWndclass.cbWndExtra    = 0;
      aWndclass.hInstance     = hInstance;
      aWndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
      aWndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
      aWndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
	   aWndclass.lpszMenuName  = NULL;
      aWndclass.lpszClassName = szWindowclassname;

      RegisterClass (&aWndclass);
	}

   hWindow = CreateWindow (szWindowclassname, "Timer-Demo 1",
                           WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);

   //-------------------------------------------------------------------------//
   // mit der Funktion SetTimer kann man bei Windows den Wunsch anmelden, da //
   // Windows in bestimmten Zeitintervallen eine genau definierte Message     //
   // schicken soll (ggf. unabhngig davon, ob das Programm gerade den Focus  //
   // hat oder nicht. SetTimer bentigt dazu (als Parameter) die folgenden    //
   // Angaben:                                                                //
   // 1. Wohin sollen die Messages geschickt werden (Handle des Fensters)     //
   // 2. Welche Nummer soll der Timer als ID-haben, denn es ist auch mglich  //
   //    einem Fenster mehrere Timer mit unterschiedlichen Zeitabstnden zu-  //
   //    zuweisen                                                             //
   // 3. In welchen Zeitabstnden soll der Timer kommen (in Millisekunden)    //
   // 4. normalerweise wird als Adresse NULL angegeben, die Message kommt wie //
   //    gehabt ber die Messagequeue (also nicht, wenn das Programm den      //
   //    Focus nicht hat), fr andere Manahmen kann hier die Adresse einer   //
   //    Funktion angegeben werden, die dann aufgerufen wird.                 //
   //-------------------------------------------------------------------------//
   while (!SetTimer (hWindow, ID_TIMER, 500, NULL))
   {
      //----------------------------------------------------------------------//
		// MessageBox ist eine sehr einfache und ntzliche Funktion, um dem An- //
      // wender Hinweise bzw. Fehlermeldungen zukommen zu lassen....          //
      //----------------------------------------------------------------------//
      nAntwort = MessageBox (hWindow, "Zuviele Timer!", szWindowclassname,
                             MB_ICONEXCLAMATION | MB_RETRYCANCEL);
      if (IDCANCEL == nAntwort) return FALSE;
   }

	ShowWindow (hWindow, nCmdShow);
   UpdateWindow (hWindow);

   while (GetMessage (&aMessage, NULL, 0, 0))
   {
	   TranslateMessage (&aMessage);
      DispatchMessage (&aMessage);
   }
   return (aMessage.wParam);
}

//----------------------------------------------------------------------------//
// Handler-Function der Client-Area                                           //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND hwnd,UINT message,UINT wParam,LONG lParam)
{
	static BOOL nColChange = FALSE;
   HBRUSH      hBrush ;
   HDC         hdc ;
   PAINTSTRUCT ps ;
   RECT        rc ;

   switch (message)
   {

		//----------------------------------------------------------------------//
		// diese Message kommt jetzt so hufig wie angegeben. Da wir nur einen  //
      // Timer benutzen, entfllt die Notwendigkeit, die Timer anhand ihrer   //
      // ID (wParam) zu unterscheiden.                                        //
		//----------------------------------------------------------------------//
   	case WM_TIMER :
			  //-----------------------------------------------------------------//
           // Bei jedem Wechsel piepsen, Farbwert umschalten und - damit in   //
           // neuer Farbe gemalt wird - invalidieren !                        //
			  //-----------------------------------------------------------------//
           MessageBeep (0);
           nColChange = !nColChange;
           InvalidateRect (hwnd, NULL, FALSE);
           return (0);

		//----------------------------------------------------------------------//
      // da wir im Timer die Clientarea invalidieren schickt uns Windows so-  //
      // fort die Aufforderung alles neu zu zeichnen - in Form einer WM_PAINT //
      // Message                                                              //
		//----------------------------------------------------------------------//
      case WM_PAINT :
           hdc = BeginPaint (hwnd, &ps);
			  //-----------------------------------------------------------------//
           // Gre der Client-Area feststellen, damit wir auch ein schnes   //
           // Rechteck malen knnen...                                        //
			  //-----------------------------------------------------------------//
           GetClientRect (hwnd, &rc);

  			  //-----------------------------------------------------------------//
           // Hier erzeugen wir einen farbigen Stift, um die Clientarea aus-  //
           // malen zu knnen                                                 //
			  //-----------------------------------------------------------------//
           hBrush = CreateSolidBrush (nColChange ? RGB(255,0,0) : RGB(0,0,255));
           FillRect (hdc, &rc, hBrush);
           EndPaint (hwnd, &ps);

			  //-----------------------------------------------------------------//
           // GANZ WICHTIG! da wir ein neues Resource-Objekt (hier einen      //
           // Brush) erzeugt haben, mssen wir dafr sorgen, da er auch wie- //
           // der freigegeben wird (anders als StockObjekte)                  //
			  //-----------------------------------------------------------------//
           DeleteObject (hBrush);
           return (0);

		//----------------------------------------------------------------------//
      // Wie immer bei Programmende....                                       //
		//----------------------------------------------------------------------//
      case WM_DESTROY :
			  //-----------------------------------------------------------------//
           // GANZ WICHTIG! da wir ein Resource-Objekt (hier einen Timer) fr //
           // unser Programm exklusiv erhalten haben und Windows diesen Timer //
           // nur fr uns vorhlt und verwaltet, mssen wir mitteilen, da    //
           // wir keine Timerbotschaften mehr bentigen. Und der Timer (es    //
           // gibt nur eine begrenzte Anzahl insgesamt) anderweitig verwendet //
           // werden kann                                                     //
			  //-----------------------------------------------------------------//
           KillTimer (hwnd, ID_TIMER);
           PostQuitMessage (0);
           return (0);
        }
     return DefWindowProc (hwnd, message, wParam, lParam);
     }