//****************************************************************************//
//   P11.cpp - Ermittelt einige wichtige Systemgrenzen                        //
//****************************************************************************//
#include <windows.h>
#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

//----------------------------------------------------------------------------//
// Defines aus dem Resource-Workshop einlesen, so da sie hier verwendbar sind//
//----------------------------------------------------------------------------//
#include "p11.rh"

//----------------------------------------------------------------------------//
// Dies sind eigene Windowsmessages, der Bereich unterhalb von WM_USER wird   //
// von Windows selbst bentigt                                                //
//----------------------------------------------------------------------------//
#define WMU_MYCOPY  WM_USER+1
#define WMU_MYPASTE WM_USER+2

//----------------------------------------------------------------------------//
// Prototyp der Handlerfunction, die die Messages der Client-Area verarbeitet //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);

//----------------------------------------------------------------------------//
// Prototyp der Handlerfunction, die die Messagebox "About" verarbeitet       //
//----------------------------------------------------------------------------//
BOOL FAR PASCAL AboutProc (HWND hDlg, UINT message, UINT wParam, LONG lParam);
BOOL FAR PASCAL Dlg2Proc  (HWND hDlg, UINT message, UINT wParam, LONG lParam);

//----------------------------------------------------------------------------//
// globale Variablen                                                          //
//----------------------------------------------------------------------------//
HINSTANCE hInst;

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"              //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
{
	static char szWindowclassname[] = "Calc";
   HWND        hWindow;
   MSG         aMessage;
   WNDCLASS    aWndclass;
   HBRUSH      mybr;

   mybr = CreateSolidBrush (RGB (100, 200, 155));

   if (!hPrevInstance)
   {
   	aWndclass.style         = CS_HREDRAW | CS_VREDRAW;
      aWndclass.lpfnWndProc   = WndProc;
      aWndclass.cbClsExtra    = 0;
      //----------------------------------------------------------------------//
      // Gibt die Anzahl der Verwaltungsbytes an, die zustzlich im Anschlu  //
      // an die Window-Instanz erzeugt werden sollen. Normalerweise ist       //
      // dieser Wert Null. Wird die WNDCLASS-Struktur aber wie hier dazu ver- //
      // wendet statt eines Fensters sofort einen Dialog mit dem Classname zu //
      // registrieren, so so mu der Wert auf DLGWINDOWEXTRA gesetzt werden   //
      //----------------------------------------------------------------------//
      aWndclass.cbWndExtra    = DLGWINDOWEXTRA;
      aWndclass.hInstance     = hInstance;
      //----------------------------------------------------------------------//
      // Zur Abwechslung holen wir uns einmal ein Icon von der Platte. Das    //
      // Icon heit hier "P11.ICO"                                            //
      //----------------------------------------------------------------------//
      aWndclass.hIcon         = LoadIcon (NULL, "P11");
      aWndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
      aWndclass.hbrBackground = mybr; // GetStockObject (GRAY_BRUSH);
	   aWndclass.lpszMenuName  = MAKEINTRESOURCE(IDM_MENU1);
      aWndclass.lpszClassName = szWindowclassname;

      RegisterClass (&aWndclass);
	}

   //-------------------------------------------------------------------------//
   // Die zur Laufzeit erzeugten Dialogboxen bentigen fr ihre eigene Ver-   //
   // waltung den Instanzenzhler. Dazu mu dieser global gepuffert werden    //
   //-------------------------------------------------------------------------//
   hInst = hInstance;
   DeleteObject (mybr);

   //-------------------------------------------------------------------------//
   // Statt CreateWindow rufen wir hier Create Dialog auf - und zwar den      //
   // Dialog, der genauso heit wie die Klasse                                //
   //-------------------------------------------------------------------------//
   hWindow = CreateDialog (hInstance, szWindowclassname, 0, NULL);
	ShowWindow (hWindow, nCmdShow);

   while (GetMessage (&aMessage, NULL, 0, 0))
   {
	   TranslateMessage (&aMessage);
      DispatchMessage (&aMessage);
   }
   return (aMessage.wParam);
}

//----------------------------------------------------------------------------//
// Ergebnisanzeige                                                            //
//----------------------------------------------------------------------------//
void ShowNumber (HWND hwnd, double fNumber)
{
   char szBuffer [50];

   //-------------------------------------------------------------------------//
   // Wir ndern einfach die Beschriftung des Buttons IDC_ERGEBNIS            //
   //-------------------------------------------------------------------------//
   sprintf (szBuffer, "%lf", fNumber);
   SetDlgItemText (hwnd, IDC_ERGEBNIS, szBuffer);
}

//----------------------------------------------------------------------------//
// Hier werden die Berechnungen durchgefhrt, die den Buttons entsprechen     //
//----------------------------------------------------------------------------//
double CalcIt (double fFirstNum, int nOperation, double fSecNum)
{
   switch (nOperation)
   {
      case IDC_PLUS   : return (fFirstNum +  fSecNum);
      case IDC_MINUS  : return (fFirstNum -  fSecNum);
      default         : return (0.0);
   }
}

//----------------------------------------------------------------------------//
// Handlerfunction der Client-Area                                            //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND hwnd,UINT message,UINT wParam,LONG lParam)
{
   //-------------------------------------------------------------------------//
   // Unter 16-bit Windows mu der Funktionsaufruf fr den Handler der Dialog-//
   // box "ABOUT" zuerst in einen FARPROC (32bit) Pointer bersetzt werden    //
   // Unter 32-Bit Windows ist dies nicht ntig, die Adresse bereits das      //
   // richtige Format hat                                                     //
   // FARPROC    lpAboutProc;                                                 //
   //-------------------------------------------------------------------------//
   static BOOL   bFirstNumber = TRUE;
   static int    nLastOp      = -1;
   HFONT         hFont, hoFont;
   int           nOldBk;
   COLORREF      nOldTxtCol;
   BOOL          bShowResult  = FALSE;
   static double fResult = 0.0, fFirstNum = 0.0, fSecNum = 0.0;
   int           wID, nAdd;
   HGLOBAL       hData;
   LPSTR         pData;
   char          szHelp [100] = "";
   RECT          rect;
   HDC           dc;
   PAINTSTRUCT   ps;

   switch (message)
   {
      //----------------------------------------------------------------------//
      // Dies ist die Auswertung einer selbstdefinierten Message, die aus dem //
      // Command heraus geschickt wird. Aufgbe dieser Message ist es, den     //
      // gerade in der Anzeige befindlichen Wert in die Zwischenablage zu     //
      // schreiben                                                            //
      //----------------------------------------------------------------------//
      case WMU_MYCOPY:
           //-----------------------------------------------------------------//
           // da die Zwischenablage allen Programmen gemeinsam zugnglich ist //
           // gehrt sie nicht zu einem Programm, sondern wird von Windows in //
           // einem eigenen Datensegment verwaltet. Ein Segment der bentig-  //
           // ten Gre versuchen wir zunchst von Windows zu erhalten        //
           //-----------------------------------------------------------------//
           hData = GlobalAlloc (GMEM_MOVEABLE, 100L);
           if (!hData) return (0);

           //-----------------------------------------------------------------//
           // Bevor wir mit dem Zwischenablage-Speicher arbeiten knnen, mu  //
           // man das Handle in einen Pointer umwandeln.                      //
           //-----------------------------------------------------------------//
           pData = (LPSTR)GlobalLock (hData);
           if (!pData)
           {
              GlobalFree (hData);
              return (0);
           }
           GlobalUnlock (hData);

           //-----------------------------------------------------------------//
           // Da wir den Zeiger auf den Speicherbereich von GlobalLock er-    //
           // halten haben, besorgen wir uns jetzt den Text, den wir dort     //
           // hineinschreiben wollen                                          //
           //-----------------------------------------------------------------//
           GetDlgItemText (hwnd, IDC_ERGEBNIS, pData, 45);

           //-----------------------------------------------------------------//
           // jetzt ffnen wir das Clipboard. Dadurch ist es fr alle anderen //
           // Anwendungen nicht mehr zugnglich                               //
           //-----------------------------------------------------------------//
           if (OpenClipboard (hwnd))
           {
              //--------------------------------------------------------------//
              // Den bisherigen Inhalt lassen wir lschen und schreiben dafr //
              // unseren Text hinein. Auf gar keinen Fall drfen wir ver-     //
              // gessen das CLIPBOARD wieder zu schlieen, da es sonst erst   //
              // nach Beendigung unseres Programms wieder zugnglich wre und //
              // andere Programme ggf. blockiert                              //
              //--------------------------------------------------------------//
              EmptyClipboard ();
              SetClipboardData (CF_TEXT, hData);
              CloseClipboard ();

              //--------------------------------------------------------------//
              // Weil man das Ergebnis nicht sehen kann, machen wir uns selbst//
              // zu Testzwecken eine kleine Mitteilung                        //
              //--------------------------------------------------------------//
              MessageBox (hwnd, "kopiert...", "CLIPBOARD", MB_OK |
                          MB_APPLMODAL | MB_ICONINFORMATION);
           }
           //-----------------------------------------------------------------//
           // Sollte das ganze nicht geklappt haben, geben wir das Segment    //
           // wieder frei                                                     //
           //-----------------------------------------------------------------//
           else
              GlobalFree(hData);
           return (0);

      //----------------------------------------------------------------------//
      // Dies ist die Auswertung einer selbstdefinierten Message, die aus dem //
      // Command heraus geschickt wird                                        //
      //----------------------------------------------------------------------//
      case WMU_MYPASTE:
           //-----------------------------------------------------------------//
           // Wir drfen nur einfgen, wenn auch ein Text im CLIPBOARD ge-    //
           // speichert ist                                                   //
           //-----------------------------------------------------------------//
           if (!IsClipboardFormatAvailable (CF_TEXT))
           {
              MessageBox (hwnd, "Kein Textformat", "CLIPBOARD", MB_OK |
                          MB_APPLMODAL | MB_ICONINFORMATION);
              return (0);
           }

           //-----------------------------------------------------------------//
           // jetzt ffnen wir das Clipboard. Dadurch ist es fr alle anderen //
           // Anwendungen nicht mehr zugnglich                               //
           //-----------------------------------------------------------------//
           if (OpenClipboard (hwnd))
           {
              //--------------------------------------------------------------//
              // Jetzt holen wir uns die Daten aus dem Clipboard zurck       //
              // Schlieen nicht vergessen, siehe oben !                      //
              //--------------------------------------------------------------//
              hData = GetClipboardData (CF_TEXT);
              lstrcpy (szHelp, (LPSTR)hData);
              CloseClipboard ();

              //--------------------------------------------------------------//
              // Den gerade gelesenen Text zeigen wir uns zu Testzwecken an   //
              //--------------------------------------------------------------//
              MessageBox (hwnd, szHelp, "CLIPBOARD gelesen", MB_OK |
                          MB_APPLMODAL | MB_ICONINFORMATION);
           }

           //-----------------------------------------------------------------//
           // Den Text wandeln wir wieder in eine Zahl um. Nicht-Ziffern      //
           // gehen natrlich verloren                                        //
           //-----------------------------------------------------------------//
           if (bFirstNumber)
           {
              fFirstNum = atof (szHelp);
              sprintf (szHelp, "%lf", fFirstNum);
           }
           else
           {
              fSecNum = atof (szHelp);
              sprintf (szHelp, "%lf", fFirstNum);
           }
           //-----------------------------------------------------------------//
           // Das Ergebnis mssen wir noch anzeigen lassen und das wr's dann //
           //-----------------------------------------------------------------//
           SetDlgItemText (hwnd, IDC_ERGEBNIS, szHelp);
           return (0);

      //----------------------------------------------------------------------//
      // Messages der einzelnen Dialogelemente (Controls)                     //
      //----------------------------------------------------------------------//
      case WM_COMMAND :
           nAdd = -1;
           //-----------------------------------------------------------------//
           // Tastatureingabefokus setzen                                     //
           //-----------------------------------------------------------------//
           SetFocus (hwnd);
           wID = LOWORD(wParam);
           switch (wID)
           {
              //--------------------------------------------------------------//
              // Programm ber Men beenden, dafr senden wir uns selbst eine //
              // DESTROY-Message                                              //
              //--------------------------------------------------------------//
              case IDM_MYEXIT:
                   SendMessage (hwnd, WM_DESTROY, 0, 0);
                   return (0);

              //--------------------------------------------------------------//
              // Menauswahl: Aus dem Clipboard einfgen                      //
              //--------------------------------------------------------------//
			     case IDM_MYPASTE:
                   SendMessage (hwnd, WMU_MYPASTE, 0, 0);
                   return (0);

              //--------------------------------------------------------------//
              // Menauswahl: in das CLIPBOARD kopieren                       //
              //--------------------------------------------------------------//
              case IDC_ERGEBNIS:
              case IDM_MYCOPY:
                   SendMessage (hwnd, WMU_MYCOPY, 0, 0);
                   return (0);

              //--------------------------------------------------------------//
              // Menauswahl: Windowshilfe aufrufen                           //
              //--------------------------------------------------------------//
              case IDM_MYHELP :
                   WinHelp (hwnd, "c:\\windows\\help\\windows.hlp",
                            HELP_CONTENTS, 0L);
                   return (0);

              //--------------------------------------------------------------//
              // Menauswahl: Unterdialog "About" / "ber" aufrufen           //
              //--------------------------------------------------------------//
              case IDM_MYABOUT:
                   //---------------------------------------------------------//
                   // Fr 16-Bit-Windows mu die nachfolgende Zeile durch     //
                   // diesen Dreizeiler ersetzt werden, um die 16-Bit-Adresse //
                   // in eine eindeutige 32-Bit-Adressen umzusetzen           //
                   // lpAboutProc = MakeProcInstance (AboutProc, hInst);      //
                   //---------------------------------------------------------//
 			          // DialogBox (hInst, MAKEINTRESOURCE(IDD_ABOUT), hwnd,     //
                   //            (DLGPROC)lpAboutProc);                       //
			          // FreeProcInstance (lpAboutProc);                         //
                   //---------------------------------------------------------//
			          DialogBox (hInst, MAKEINTRESOURCE(IDD_ABOUT), hwnd,
                              (DLGPROC)AboutProc);
                   return (0);

              case CM_DLG2:
			          DialogBox (hInst, MAKEINTRESOURCE(DLG2), hwnd,
                              (DLGPROC)Dlg2Proc);
                   return (0);

              //--------------------------------------------------------------//
              // Einer der Zahlenbuttons wurde gedrckt                       //
              //--------------------------------------------------------------//
              case IDC_0     : nAdd = 0; break;
              case IDC_1     : nAdd = 1; break;
              case IDC_2     : nAdd = 2; break;
              case IDC_3     : nAdd = 3; break;
              case IDC_4     : nAdd = 4; break;
              case IDC_5     : nAdd = 5; break;
              case IDC_6     : nAdd = 6; break;
              case IDC_7     : nAdd = 7; break;
              case IDC_8     : nAdd = 8; break;
              case IDC_9     : nAdd = 9; break;

              //--------------------------------------------------------------//
              // Programm in den Grundzustand bringen                         //
              //--------------------------------------------------------------//
              case IDC_CLEAR :
   					 bFirstNumber = TRUE;
   					 nLastOp      = -1;
                   fFirstNum    = 0.0,
                   fSecNum      = 0.0;
                   nAdd         = 0;
                   break;

              //--------------------------------------------------------------//
              // Einer der Operatorbuttons wurde gedrckt                     //
              //--------------------------------------------------------------//
              default :
                   if (bFirstNumber) bFirstNumber = FALSE;
                   else
                   {
                      bShowResult = TRUE;
                      if (wID == IDC_EQUALS)
                         fResult = CalcIt (fFirstNum, nLastOp, fSecNum);
                      else
                         fResult = CalcIt (fFirstNum, wID, fSecNum);
                   }
                   nLastOp = wID;
                   break;
           }

           //-----------------------------------------------------------------//
           // Eingaben der Zahlen/Operatoren auswerten und anzeigen           //
           //-----------------------------------------------------------------//
           if (bShowResult)
           {
              ShowNumber (hwnd, fResult);
              fFirstNum = fResult;
              fResult = 0.0;
              fSecNum = 0.0;
              bFirstNumber = FALSE;
           }
           else
           {
              if ((bFirstNumber) && (nAdd != -1))
              {
                 fFirstNum = fFirstNum * 10 + nAdd;
                 ShowNumber (hwnd, fFirstNum);
              }
              if ((!bFirstNumber) && (nAdd != -1))
              {
                 fSecNum = fSecNum * 10 + nAdd;
                 ShowNumber (hwnd, fSecNum);
              }
           }
           return (0);

      //----------------------------------------------------------------------//
      // Zeichnen der zustzlichen Elemente                                   //
      //----------------------------------------------------------------------//
      case WM_PAINT:
           //-----------------------------------------------------------------//
           // Rechteck der Client-Area ermitteln                              //
           //-----------------------------------------------------------------//
           GetClientRect (hwnd, &rect);
           rect.left += 5;
           rect.top  += 10;
           rect.right -= 5;
           rect.bottom = rect.top + 30;

           //-----------------------------------------------------------------//
           // Einen True-Type-Font in gewnschter Ausprgung erzeugen         //
           //-----------------------------------------------------------------//
           hFont  = CreateFont (30, 0, 0, 0, FW_THIN, TRUE, FALSE, FALSE,
                                ANSI_CHARSET, OUT_DEFAULT_PRECIS,
                                CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
                                DEFAULT_PITCH, "Braggadocio");

           //-----------------------------------------------------------------//
           // Ermittlung des Device-Contextes und Auswahl der "Mal-Utensilien"//
           // dabei die vorherigen Zustnde merken, um sie hinterher sauber   //
           // wiederherzustellen                                              //
           //-----------------------------------------------------------------//
           dc         = BeginPaint (hwnd, &ps);
           hoFont     = SelectObject (dc, hFont);
           nOldBk     = GetBkMode (dc);
           nOldTxtCol = GetTextColor (dc);
           SetBkMode (dc, TRANSPARENT);
           SetTextColor (dc, RGB (255,255,0));
           DrawText (dc, "Calculator", 10, &rect, 17);
           SetTextColor (dc, nOldTxtCol);
           SetBkMode    (dc, nOldBk);
           SelectObject (dc, hoFont);
           DeleteObject (hFont);
           EndPaint (hwnd, &ps);
           return (DefWindowProc (hwnd, message, wParam, lParam));

      //----------------------------------------------------------------------//
      // Beenden des Programms                                                //
      //----------------------------------------------------------------------//
      case WM_DESTROY :
           PostQuitMessage (0);
           return (0);
   }
   return DefWindowProc (hwnd, message, wParam, lParam);
}

//----------------------------------------------------------------------------//
// Handlerfunction der About-Box                                              //
//----------------------------------------------------------------------------//
#pragma argsused
BOOL FAR PASCAL AboutProc (HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
   switch (message)
   {
      //----------------------------------------------------------------------//
      // Initialisierung der Box (falls ntig)                                //
      //----------------------------------------------------------------------//
      case WM_INITDIALOG:
           break;

      //----------------------------------------------------------------------//
      // Hier den einzigen Button abfragen und dann den Dialog beenden        //
      //----------------------------------------------------------------------//
	   case WM_COMMAND:
		     switch (wParam)
		     {
		        case IDOK:
			          EndDialog (hDlg, TRUE);
			          break;
           }
		     break;
   }
   return (FALSE);
}

//----------------------------------------------------------------------------//
// Handlerfunction der About-Box                                              //
//----------------------------------------------------------------------------//
#pragma argsused
BOOL FAR PASCAL Dlg2Proc (HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
	char szBuffer [200];

   switch (message)
   {
      //----------------------------------------------------------------------//
      // Initialisierung der Box (falls ntig)                                //
      //----------------------------------------------------------------------//
      case WM_INITDIALOG:
           GetPrivateProfileString ("Register",
              "Reg", "Nix", (LPSTR)szBuffer, 200,
              "E:\\27275\\my.ini");

           SendDlgItemMessage (hDlg, IDC_EDIT2, EM_LIMITTEXT, 15, 0);
           SetWindowText (GetDlgItem (hDlg, IDC_EDIT2),
                          (LPSTR)szBuffer);
           break;

      //----------------------------------------------------------------------//
      // Hier den einzigen Button abfragen und dann den Dialog beenden        //
      //----------------------------------------------------------------------//
	   case WM_COMMAND:
		     switch (wParam)
		     {
		        case IDOK:
                   GetWindowText(GetDlgItem (hDlg, IDC_EDIT2),
                    (LPSTR)szBuffer, 200);
		             WritePrivateProfileString ("Register",
                    "Reg", (LPSTR)szBuffer,
                    "E:\\27275\\my.ini");
			          EndDialog (hDlg, TRUE);
			          break;
           }
		     break;
   }
   return (FALSE);
}


