//****************************************************************************//
//   P13.cpp - einfache Druckausgabe                                          //
//****************************************************************************//
#include <windows.h>
#include <string.h>
#include <stdio.h>

//----------------------------------------------------------------------------//
// Prototypen                                                                 //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);
BOOL    PrintMyPage  (HWND);
HDC     GetPrinterDC (void);
void    PageGDICalls (HDC, int, int);

//----------------------------------------------------------------------------//
// globale Variablen                                                          //
//----------------------------------------------------------------------------//
HANDLE hInst;
char   szWindowclassname[] = "P13";
short  cxClient, cyClient;

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"              //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
{
   HWND        hWindow;
   MSG         aMessage;
   WNDCLASS    aWndclass;

   if (!hPrevInstance)
   {
	   aWndclass.style         = CS_HREDRAW | CS_VREDRAW;
   	aWndclass.lpfnWndProc   = WndProc;
	   aWndclass.cbClsExtra    = 0;
   	aWndclass.cbWndExtra    = 0;
	   aWndclass.hInstance     = hInstance;
   	aWndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
	   aWndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
   	aWndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
	   aWndclass.lpszMenuName  = NULL;
   	aWndclass.lpszClassName = szWindowclassname;

	   RegisterClass (&aWndclass);
   }

   hInst = hInstance;

   hWindow = CreateWindow (szWindowclassname, "Druck-Demo",
                           WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);

	ShowWindow   (hWindow, nCmdShow);
   UpdateWindow (hWindow);

   while (GetMessage (&aMessage, NULL, 0, 0))
   {
	   TranslateMessage (&aMessage);
      DispatchMessage  (&aMessage);
   }
   return (aMessage.wParam);
}

//----------------------------------------------------------------------------//
// Device-Context des Druckers (Treiber, Auflsung etc.) ermitteln            //
//----------------------------------------------------------------------------//
HDC GetPrinterDC (void)
{
   static char szPrinter [80];
   char        *szDevice, *szDriver, *szOutput;

	//-------------------------------------------------------------------------//
   // Daten aus der Ini-Datei lesen                                           //
	//-------------------------------------------------------------------------//
   GetProfileString ("windows", "device", ",,,", szPrinter, 80);
   MessageBox (NULL, szPrinter, "Drucker:", MB_OK | MB_TASKMODAL);

	//-------------------------------------------------------------------------//
   // Wenn die Daten korrekt aus der INI-Datei gelesen werden konnten, dann   //
   // den entsprechenden Device-Context erzeugen                              //
	//-------------------------------------------------------------------------//
   if (NULL != (szDevice = strtok (szPrinter, "," )) &&
       NULL != (szDriver = strtok (NULL,      ", ")) &&
       NULL != (szOutput = strtok (NULL,      ", ")))
   {
		return CreateDC (szDriver, szDevice, szOutput, NULL);
   }
   return (0);
}

//----------------------------------------------------------------------------//
// Aufrufe an da GDI auf der Drucker schicken                                 //
//----------------------------------------------------------------------------//
void PageGDICalls (HDC hdcPrn, int cxPage, int cyPage)
{
   static char szTextStr [] = "Hello, Printer!";

	//-------------------------------------------------------------------------//
   // Statt auf dem Bildschirm zeichnen wir einfach auf dem Drucker           //
	//-------------------------------------------------------------------------//
   Rectangle (hdcPrn, 0, 0, cxPage, cyPage);

   MoveToEx (hdcPrn, 0, 0, NULL);
   LineTo   (hdcPrn, cxPage, cyPage);
   MoveToEx (hdcPrn, cxPage, 0, NULL);
   LineTo   (hdcPrn, 0, cyPage);

	//-------------------------------------------------------------------------//
   // Die aktuellen Einstellungen fr den Druckerkontext sichern              //
	//-------------------------------------------------------------------------//
   SaveDC   (hdcPrn);

	//-------------------------------------------------------------------------//
   // Skalierungsverfahren fr die Druckausgabe ndern                        //
	//-------------------------------------------------------------------------//
   SetMapMode       (hdcPrn, MM_ISOTROPIC);

	//-------------------------------------------------------------------------//
   // "Fenstergre" fr die Ausgabe festlegen                                //
	//-------------------------------------------------------------------------//
   SetWindowExtEx   (hdcPrn, 1000, 1000, NULL);
   SetViewportExtEx (hdcPrn, cxPage / 2, -cyPage / 2, NULL);
   SetViewportOrgEx (hdcPrn, cxPage / 2,  cyPage / 2, NULL);

	//-------------------------------------------------------------------------//
   // Ausgaben                                                                //
	//-------------------------------------------------------------------------//
   Ellipse (hdcPrn, -500, 500, 500, -500);
   SetTextAlign (hdcPrn, TA_BASELINE | TA_CENTER);
   TextOut (hdcPrn, 0, 0, szTextStr, sizeof szTextStr - 1);

	//-------------------------------------------------------------------------//
   // Einstellungen des Device-Contextes wiederherstellen                     //
	//-------------------------------------------------------------------------//
   RestoreDC (hdcPrn, -1);
}

//----------------------------------------------------------------------------//
// Handlerfunktion der Client-Area                                            //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND hwnd,UINT message,UINT wParam,LONG lParam)
{
   HDC          hdc;
   HMENU        hMenu;
   PAINTSTRUCT  ps;

   switch (message)
   {
      //----------------------------------------------------------------------//
      // Beim Erzeugen der Client-Area (Initialisierung)                      //
      //----------------------------------------------------------------------//
   	case WM_CREATE:
           hMenu = GetSystemMenu (hwnd, FALSE);
           AppendMenu (hMenu, MF_SEPARATOR, 0, NULL);
           AppendMenu (hMenu, 0, 1, "&Drucken");
           return (0);

      //----------------------------------------------------------------------//
      // Bei Vernderung der Gre des Fensters                               //
      //----------------------------------------------------------------------//
      case WM_SIZE:
           cxClient = LOWORD (lParam);
           cyClient = HIWORD (lParam);
           return (0);

      //----------------------------------------------------------------------//
      // Bei Auswahl im System-Men                                           //
      //----------------------------------------------------------------------//
      case WM_SYSCOMMAND:
           if (wParam == 1)
           {
              if (PrintMyPage (hwnd))
              {
                 MessageBox (hwnd, "Fehler beim Drucken der Seite!",
                             szWindowclassname, MB_OK | MB_ICONEXCLAMATION);
              }
              return (0);
                    }
           break;

      //----------------------------------------------------------------------//
      // Beim Neuzeichnen des Fensters                                        //
      //----------------------------------------------------------------------//
      case WM_PAINT:
           hdc = BeginPaint (hwnd, &ps);

           //-----------------------------------------------------------------//
           // Aufruf der Mal-Funktion "PageGDICalls" mit dem Display-Context  //
           // mit Angabe der Bildschirmauflsung                              //
           //-----------------------------------------------------------------//
           PageGDICalls (hdc, cxClient, cyClient);
           EndPaint (hwnd, &ps);
           return (0);

      //----------------------------------------------------------------------//
      // Beim Verlassen des Programms                                         //
      //----------------------------------------------------------------------//
      case WM_DESTROY:
           PostQuitMessage (0);
           return (0);
   }
   return DefWindowProc (hwnd, message, wParam, lParam);
}

//----------------------------------------------------------------------------//
// Ausgelagerte "Malfunktion" fr den Drucker, ruft genau wie WM_PAINT die    //
// Funktion PageGDICalls auf                                                  //
//----------------------------------------------------------------------------//
#pragma argsused
BOOL PrintMyPage (HWND hwnd)
{
   HDC hdcPrn;
   DOCINFO     di;
   int         xPage, yPage;

	//-------------------------------------------------------------------------//
   // Device-Context des Druckers holen, wenn keiner verfgbar ist, dann mit  //
   // Fehler zurck                                                           //
	//-------------------------------------------------------------------------//
   hdcPrn = GetPrinterDC ();
   if (NULL == hdcPrn) return TRUE ;

	//-------------------------------------------------------------------------//
   // Auflsung des Druckers ermitteln                                        //
	//-------------------------------------------------------------------------//
   xPage = GetDeviceCaps (hdcPrn, HORZRES) ;
   yPage = GetDeviceCaps (hdcPrn, VERTRES) ;

	//-------------------------------------------------------------------------//
   // DocInfo-Struktur fr die Drucker-Queue ausfllen                        //
	//-------------------------------------------------------------------------//
   ZeroMemory (&di, sizeof (DOCINFO));
   di.cbSize = sizeof (DOCINFO);
   di.lpszDocName = "TestDoc";

	//-------------------------------------------------------------------------//
   // Druckvorgang mit neuer Seite starten                                    //
	//-------------------------------------------------------------------------//
   StartDoc (hdcPrn, &di);
   StartPage (hdcPrn);

	//-------------------------------------------------------------------------//
   // Hier mach wir das gleiche, wie auf dem Bildschirm...                    //
	//-------------------------------------------------------------------------//
   PageGDICalls (hdcPrn, xPage, yPage);

	//-------------------------------------------------------------------------//
   // Seite und Druckvorgang beenden, Device-Context wieder freigeben         //
	//-------------------------------------------------------------------------//
   EndPage (hdcPrn);
   EndDoc  (hdcPrn);
   DeleteDC(hdcPrn);

   return (0);
}

