//****************************************************************************//
//   P6.cpp - Kontrollelemente-Demo                                           //
//****************************************************************************//
#include <windows.h>
#include <stdio.h>

//----------------------------------------------------------------------------//
// globale Struktur fr Kontrollelemente                                      //
//----------------------------------------------------------------------------//
struct { long style; char *text; } button [] =
       {
          {BS_PUSHBUTTON,      "PUSHBUTTON"   },
          {BS_DEFPUSHBUTTON,   "DEFPUSHBUTTON"},
     		 {BS_CHECKBOX,        "CHECKBOX"     },
     		 {BS_AUTOCHECKBOX,    "AUTOCHECKBOX" },
     		 {BS_RADIOBUTTON,     "RADIOBUTTON"  },
     		 {BS_3STATE,          "3STATE"       },
     		 {BS_AUTO3STATE,      "AUTO3STATE"   },
     		 {BS_GROUPBOX,        "GROUPBOX"     },
     		 {BS_AUTORADIOBUTTON, "AUTORADIO"    },
     		 {BS_OWNERDRAW,       "OWNERDRAW"    }
       };
#define NUM (sizeof button / sizeof button [0])

//----------------------------------------------------------------------------//
// Prototyp der Handlerfunction, die die Messages der Client-Area verarbeitet //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"              //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
{
   //-------------------------------------------------------------------------//
	// Hier wieder die vier unverzichtbaren Variablen...                       //
   //-------------------------------------------------------------------------//
	static char szWindowclassname [] = "P6";
   HWND        hWindow;
   MSG         aMessage;
   WNDCLASS    aWindowclass;

   if (!hPrevInstance)
   {
   	aWindowclass.style         = CS_HREDRAW | CS_VREDRAW;
      aWindowclass.lpfnWndProc   = WndProc;
      aWindowclass.cbClsExtra    = 0;
      aWindowclass.cbWndExtra    = 0;
      aWindowclass.hInstance     = hInstance;
      aWindowclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
      aWindowclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
      aWindowclass.hbrBackground = GetStockObject (WHITE_BRUSH);
      aWindowclass.lpszMenuName  = NULL;
      aWindowclass.lpszClassName = szWindowclassname;

	   RegisterClass (&aWindowclass);
   }
   //-------------------------------------------------------------------------//
	// Fenster erzeugen und anzeigen                                           //
   //-------------------------------------------------------------------------//
   hWindow = CreateWindow (szWindowclassname, "Kontrollelemente",
                           WS_OVERLAPPEDWINDOW,
      							CW_USEDEFAULT, CW_USEDEFAULT,
      							CW_USEDEFAULT, CW_USEDEFAULT,
      							NULL, NULL, hInstance, NULL);
	ShowWindow   (hWindow, nCmdShow);
   UpdateWindow (hWindow);

   //-------------------------------------------------------------------------//
	// Verarbeiten aller Botschaften an unser Programm                         //
   //-------------------------------------------------------------------------//
   while (GetMessage (&aMessage, NULL, 0, 0))
   {
   	TranslateMessage (&aMessage);
      DispatchMessage  (&aMessage);
   }
   //-------------------------------------------------------------------------//
	// Windowskernel mitteilen, da die WM_QUIT-Message verarbeitet wurde      //
   //-------------------------------------------------------------------------//
   return (aMessage.wParam);
}

//----------------------------------------------------------------------------//
// Messagehandler der Client-Area                                             //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND hwnd,UINT message,UINT wParam,LONG lParam)
{
   static char  szTop []    = "Botschaft     wParam         lParam";
	static char  szUnd []    = "___________   _____________  ____________";
   static char  szFormat [] = "%-16s%6X%8X-%04X";
   static char  szBuffer [50];
   static HWND  hwndButton [NUM];
   static RECT  rect;
   static int   cxChar, cyChar;
   HDC          hdc;
   PAINTSTRUCT  ps;
   int          i;
   TEXTMETRIC   tm;

   switch (message)
   {
   	//----------------------------------------------------------------------//
		// Initialisierung des Fensters                                         //
   	//----------------------------------------------------------------------//
   	case WM_CREATE :
           hdc = GetDC (hwnd);
           //-----------------------------------------------------------------//
           // durchschnittliche Zeichengren ermitteln                       //
           //-----------------------------------------------------------------//
           SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT));
           GetTextMetrics (hdc, &tm);
           cxChar = tm.tmAveCharWidth;
           cyChar = tm.tmHeight + tm.tmExternalLeading;
           ReleaseDC (hwnd, hdc);

           //-----------------------------------------------------------------//
           // Dialogelemente (hier Buttons) erzeugen. Auch dies sind nur      //
           // Child-Fenster, allerdings mit jeweils genau festgelegten,       //
           // speziellen Eigenschaften                                        //
           //-----------------------------------------------------------------//
           for (i = 0; i < NUM; i++)
           {
              hwndButton [i] = CreateWindow ("button", button[i].text,
                               WS_CHILD | WS_VISIBLE | button[i].style,
                               cxChar, cyChar * (1 + 2 * i),
                               20 * cxChar, 7 * cyChar / 4,
                               hwnd, (HMENU)i,
                               (HANDLE)((LPCREATESTRUCT) lParam) -> hInstance,
                               NULL);
           }
           return (0);

   	//----------------------------------------------------------------------//
		// Gre des Fensters wird verndert                                    //
   	//----------------------------------------------------------------------//
      case WM_SIZE :
           rect.left   = 24 * cxChar;
           rect.top    =  3 * cyChar;
           rect.right  = LOWORD (lParam);
           rect.bottom = HIWORD (lParam);
           return (0);

   	//----------------------------------------------------------------------//
		// Neuzeichnen des Fensters                                             //
   	//----------------------------------------------------------------------//
      case WM_PAINT :
           InvalidateRect (hwnd, &rect, TRUE);
           hdc = BeginPaint (hwnd, &ps);
           SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT));
           SetBkMode (hdc, TRANSPARENT);
	        TextOut (hdc, 24 * cxChar, 2 * cyChar, szTop, sizeof szTop - 1);
           TextOut (hdc, 24 * cxChar, 2 * cyChar, szUnd, sizeof szUnd - 1);
           EndPaint (hwnd, &ps);
           return (0);

   	//----------------------------------------------------------------------//
		// Wird ein Dialogelement bettigt, so erhlt man eine WM_COMMNAD-      //
      // Message. Man mu dann normalerweise erst feststellen, welches Dialog-//
      // element bettigt und was damit gemacht wurde                         //
      // Die DRAWITEM-Message kommt, wenn ein visiueller Aspekt des Dialog-   //
      // Fenster gendert wurde. Diese Message ist insbesondere Dann wichtig, //
      // wenn man selbstgestaltete Dialogelemente einsetzt                    //
   	//----------------------------------------------------------------------//
      case WM_COMMAND :
      case WM_DRAWITEM:
           ScrollWindow (hwnd, 0, -cyChar, &rect, &rect);
           hdc = GetDC (hwnd);
           SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT));
           sprintf (szBuffer, szFormat, (LPSTR)
			           (message == WM_COMMAND? "WM_COMMAND" : "WM_DRAWITEM"),
			           wParam, HIWORD (lParam), LOWORD (lParam));
           TextOut (hdc, 24 * cxChar, cyChar * (rect.bottom / cyChar - 1),
			           szBuffer, strlen (szBuffer));
           ReleaseDC (hwnd, hdc);
           ValidateRect (hwnd, NULL);
           return (0);

   	//----------------------------------------------------------------------//
      // Beenden des Programms                                                //
   	//----------------------------------------------------------------------//
      case WM_DESTROY :
           PostQuitMessage (0);
           return (0);
   }
   return DefWindowProc (hwnd, message, wParam, lParam);
}
