//****************************************************************************//
//   P6c.cpp - Timer-Demo 2                                                   //
//****************************************************************************//
#include <windows.h>

#define ID_TIMER    1

//----------------------------------------------------------------------------//
// Prototyp der Handlerfunction, die die Messages der Client-Area verarbeitet //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc   (HWND, UINT, UINT, LONG);

//----------------------------------------------------------------------------//
// Prototyp der Callback-Function des Timers                                  //
//----------------------------------------------------------------------------//
UINT FAR PASCAL _export TimerProc (HWND, UINT, UINT, LONG);

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"              //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
{
	static char szWindowclassname[] = "P6c";
   HWND        hWindow;
   MSG         aMessage;
   WNDCLASS    aWndclass;
   int         nAntwort;

	//-------------------------------------------------------------------------//
   // Hier speichern wir die Adresse der Timerfunction                        //
	//-------------------------------------------------------------------------//
   FARPROC     lpfnTimerProc;

   if (!hPrevInstance)
   {
   	aWndclass.style         = CS_HREDRAW | CS_VREDRAW;
      aWndclass.lpfnWndProc   = WndProc;
      aWndclass.cbClsExtra    = 0;
      aWndclass.cbWndExtra    = 0;
      aWndclass.hInstance     = hInstance;
      aWndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
      aWndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
      aWndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
	   aWndclass.lpszMenuName  = NULL;
      aWndclass.lpszClassName = szWindowclassname;

      RegisterClass (&aWndclass);
	}

   hWindow = CreateWindow (szWindowclassname, "Timer-Demo 2",
                           WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);

   //-------------------------------------------------------------------------//
   // mit der Funktion SetTimer kann man bei Windows den Wunsch anmelden, da //
   // Windows in bestimmten Zeitintervallen eine genau definierte Message     //
   // schicken soll (ggf. unabhngig davon, ob das Programm gerade den Focus  //
   // hat oder nicht. SetTimer bentigt dazu (als Parameter) die folgenden    //
   // Angaben:                                                                //
   // 1. Wohin sollen die Messages geschickt werden (Handle des Fensters)     //
   // 2. Welche Nummer soll der Timer als ID-haben, denn es ist auch mglich  //
   //    einem Fenster mehrere Timer mit unterschiedlichen Zeitabstnden zu-  //
   //    zuweisen                                                             //
   // 3. In welchen Zeitabstnden soll der Timer kommen (in Millisekunden)    //
   // 4. Die Adresse der Function TimerProc wird in eine Instanz-umgewandelt, //
   //    damit Windows sich merken kann, wie oft diese Function in Gebrauch   //
   //    ist (wichtig, falls die Function aus einer DLL kommt, da es dann     //
   //    von der Anzahl der Instanzen abhngt, ob die DLL aus dem Speicher    //
   //    entfernt werden kann                                                 //
   //-------------------------------------------------------------------------//
   lpfnTimerProc = MakeProcInstance ((FARPROC) TimerProc, hInstance);

   while (!SetTimer (hWindow, ID_TIMER, 500, lpfnTimerProc))
   {
      //----------------------------------------------------------------------//
		// MessageBox ist eine sehr einfache und ntzliche Funktion, um dem An- //
      // wender Hinweise bzw. Fehlermeldungen zukommen zu lassen....          //
      //----------------------------------------------------------------------//
      nAntwort = MessageBox (hWindow, "Zuviele Timer!", szWindowclassname,
                             MB_ICONEXCLAMATION | MB_RETRYCANCEL);
      if (IDCANCEL == nAntwort) return FALSE;
   }

	ShowWindow (hWindow, nCmdShow);
   UpdateWindow (hWindow);

   while (GetMessage (&aMessage, NULL, 0, 0))
   {
	   TranslateMessage (&aMessage);
      DispatchMessage (&aMessage);
   }
   return (aMessage.wParam);
}


//----------------------------------------------------------------------------//
// Handler-Function der Client-Area                                           //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND hwnd,UINT message,UINT wParam,LONG lParam)
{
   switch (message)
   {
		//----------------------------------------------------------------------//
      // Wie immer bei Programmende....                                       //
		//----------------------------------------------------------------------//
      case WM_DESTROY :
			  //-----------------------------------------------------------------//
           // GANZ WICHTIG! da wir ein Resource-Objekt (hier einen Timer) fr //
           // unser Programm exklusiv erhalten haben und Windows diesen Timer //
           // nur fr uns vorhlt und verwaltet, mssen wir mitteilen, da    //
           // wir keine Timerbotschaften mehr bentigen. Und der Timer (es    //
           // gibt nur eine begrenzte Anzahl insgesamt) anderweitig verwendet //
           // werden kann                                                     //
			  //-----------------------------------------------------------------//
           KillTimer (hwnd, ID_TIMER);
           PostQuitMessage (0);
           return (0);
   }
   return DefWindowProc (hwnd, message, wParam, lParam);
}

//----------------------------------------------------------------------------//
// Handler-Function des Timers                                                //
//----------------------------------------------------------------------------//
#pragma argsused
UINT FAR PASCAL _export TimerProc (HWND hwnd, UINT message, UINT wParam,
                                   LONG lParam)
{
	static BOOL nColChange = FALSE;
   HBRUSH      hBrush;
   HDC         hdc;
   RECT        rc;

	//-------------------------------------------------------------------------//
   // da diese Funktion automatisch bei einer Timerbotschaft aufgerufen wird, //
   // ist hier nur ggf. eine Fallunterscheidung der Timer ntig (Falls alle   //
   // unterschiedlichen Timer ber die gleiche TimerProc laufen)              //
	//-------------------------------------------------------------------------//
   MessageBeep (0);
   nColChange = !nColChange;
   GetClientRect (hwnd, &rc);
   hdc = GetDC (hwnd);

	//-------------------------------------------------------------------------//
   // Hier erzeugen wir einen farbigen Stift, um die Clientarea auszumalen    //
	//-------------------------------------------------------------------------//
   hBrush = CreateSolidBrush (nColChange ? RGB(255,0,0) : RGB(0,0,255));
   FillRect (hdc, &rc, hBrush);
   ReleaseDC (hwnd, hdc);

	//-------------------------------------------------------------------------//
   // GANZ WICHTIG! da wir ein neues Resource-Objekt (hier einen Brush) er-   //
   // zeugt haben, mssen wir dafr sorgen, da er auch wieder freigegeben    //
   // wird (anders als StockObjekte)                                          //
	//-------------------------------------------------------------------------//
   DeleteObject (hBrush);

	//-------------------------------------------------------------------------//
   // Auch hier die Return-Meldung an Windows, da die Botschaft erledigt ist //
	//-------------------------------------------------------------------------//
   return (0);
}

