//****************************************************************************//
//   P10.cpp - Selbstdefinierte Eingabeelemente                               //
//****************************************************************************//
#include <windows.h>
#include <stdio.h>

#define IDC_SMALLER   1
#define IDC_LARGER    2

//----------------------------------------------------------------------------//
// Prprozessor Makros                                                        //
//----------------------------------------------------------------------------//
#define BTN_WIDTH     (8 * cxChar)
#define BTN_HEIGHT    (4 * cyChar)

//----------------------------------------------------------------------------//
// globale Variablen                                                          //
//----------------------------------------------------------------------------//
HANDLE hInst;

//----------------------------------------------------------------------------//
// Prototypen der Handlerfunktionen                                           //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);

//----------------------------------------------------------------------------//
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"              //
//----------------------------------------------------------------------------//
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
{
	static char szWindowclassname[] = "P10";
   HWND        hWindow;
   MSG         aMessage;
   WNDCLASS    aWndclass;

   hInst = hInstance;

   if (!hPrevInstance)
   {
	   aWndclass.style         = CS_HREDRAW | CS_VREDRAW;
   	aWndclass.lpfnWndProc   = WndProc;
	   aWndclass.cbClsExtra    = 0;
   	aWndclass.cbWndExtra    = 0;
	   aWndclass.hInstance     = hInstance;
   	aWndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
	   aWndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
   	aWndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
	   aWndclass.lpszMenuName  = NULL;
   	aWndclass.lpszClassName = szWindowclassname;

   	RegisterClass (&aWndclass);
   }

   hWindow = CreateWindow (szWindowclassname, "Eigene Kontrollelemente",
                           WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);

	ShowWindow (hWindow, nCmdShow);
   UpdateWindow (hWindow);

   while (GetMessage (&aMessage, NULL, 0, 0))
   {
      TranslateMessage (&aMessage);
      DispatchMessage  (&aMessage);
   }
   return (aMessage.wParam);
}

//----------------------------------------------------------------------------//
// Funktion zum Zeichnen eines Dreiecks                                       //
//----------------------------------------------------------------------------//
void Triangle(HDC hdc, POINT pt[])
{
   SelectObject (hdc, GetStockObject (BLACK_BRUSH));
   Polygon (hdc, pt, 3);
   SelectObject (hdc, GetStockObject (WHITE_BRUSH));
}

//----------------------------------------------------------------------------//
// Handler der Client-Area                                                    //
//----------------------------------------------------------------------------//
long FAR PASCAL _export WndProc(HWND hwnd,UINT message,UINT wParam,LONG lParam)
{
   static HWND       hwndSmaller, hwndLarger;
   static short      cxClient, cyClient, cxChar, cyChar;
   LPDRAWITEMSTRUCT  lpdis;
   POINT             pt[3];
   RECT              rc;
   int               cx, cy;

   switch (message)
   {
      //----------------------------------------------------------------------//
      // Fensterinitialisierung                                               //
      //----------------------------------------------------------------------//
      case WM_CREATE :
           //-----------------------------------------------------------------//
           // Ermitteln der Dialoggre in BASEUNITS. Die horizontale Base-   //
           // unit entspricht der durchschnittlichen Breite des Systemfonts   //
           // (in Pixeln). Die vertikale Baseunit entspricht der Hhe des     //
           // Systemfonts (in Pixeln).                                        //
           //-----------------------------------------------------------------//
           cxChar = LOWORD (GetDialogBaseUnits ());
           cyChar = HIWORD (GetDialogBaseUnits ());

           //-----------------------------------------------------------------//
           // Erzeugen der selbstdefinierten Elemente                         //
           // Da wir die Elemente selber zeichnen mssen, erhalten die Dialog-//
           // elemente den Buttonstil BS_OWNERDRAW. Wir erhalten dann eine    //
           // WM_DRAWITEM-Message, wenn ein Dialogelement neu gezeichnet wer- //
           // den mu                                                         //
           //-----------------------------------------------------------------//
           hwndSmaller = CreateWindow("button", "",
                         WS_CHILD | WS_VISIBLE | BS_OWNERDRAW,
                         0, 0, BTN_WIDTH, BTN_HEIGHT,
                         hwnd, (HMENU)IDC_SMALLER, hInst, 0);
           hwndLarger  = CreateWindow("button", "",
                         WS_CHILD | WS_VISIBLE | BS_OWNERDRAW,
                         0, 0, BTN_WIDTH, BTN_HEIGHT,
                         hwnd, (HMENU)IDC_LARGER, hInst, 0);
           return (0);

      //----------------------------------------------------------------------//
      // Bei Grenvernderung des Fensters                                   //
      //----------------------------------------------------------------------//
      case WM_SIZE :
           //-----------------------------------------------------------------//
           // neue Gre aus den Parametern ermitteln                         //
           //-----------------------------------------------------------------//
           cxClient = LOWORD (lParam);
           cyClient = HIWORD (lParam);

           //-----------------------------------------------------------------//
           // Selbstdefinierte Elemente wieder in die Mitte rcken            //
           //-----------------------------------------------------------------//
           MoveWindow (hwndSmaller, cxClient / 2 - 3 * BTN_WIDTH / 2,
                                    cyClient / 2 -     BTN_HEIGHT / 2,
                                    BTN_WIDTH, BTN_HEIGHT, TRUE);
           MoveWindow (hwndLarger,  cxClient / 2 +     BTN_WIDTH / 2,
                                    cyClient / 2 -     BTN_HEIGHT / 2,
                                    BTN_WIDTH, BTN_HEIGHT, TRUE);
           return (0);

      //----------------------------------------------------------------------//
      // Benutzerbefehle / Eingaben an/in Dialogelementen                     //
      //----------------------------------------------------------------------//
 	   case WM_COMMAND :
           GetWindowRect (hwnd, &rc);

           //-----------------------------------------------------------------//
           // Fenster um 10% vergrern bzw. verkleinern                      //
           //-----------------------------------------------------------------//
           switch (wParam)
           {
              //--------------------------------------------------------------//
              // "Kleiner" bettigt                                           //
              //--------------------------------------------------------------//
              case IDC_SMALLER:
                   rc.left   += cxClient / 20;
                   rc.right  -= cxClient / 20;
                   rc.top    += cyClient / 20;
                   rc.bottom -= cyClient / 20;
                   break;

              //--------------------------------------------------------------//
              // "Grer"  bettigt                                           //
              //--------------------------------------------------------------//
              case IDC_LARGER:
                   rc.left   -= cxClient / 20;
                   rc.right  += cxClient / 20;
                   rc.top    -= cyClient / 20;
                   rc.bottom += cyClient / 20;
                   break;
           }
           MoveWindow (hwnd, rc.left, rc.top, rc.right - rc.left,
                       rc.bottom - rc.top, TRUE);
           return (0);

      //----------------------------------------------------------------------//
      // Die DRAWITEM-Message wird gesendet, wenn es ntig ist, ein Dialog-   //
      // element neu zu zeichnen (z.B. weil es verdeckt war)                  //
      //----------------------------------------------------------------------//
      case WM_DRAWITEM:
           //-----------------------------------------------------------------//
           // Die DRAWITEMSTRUCT enthlt weitergehende Angaben dazu, was zu   //
           // zu zeichnen ist (gesamtes Element, wurde Element deselektiert   //
           // usw.) und welchen Zustand des Objekt angenommen hat (hat es den //
           // Fokus bekommen, wurde es abgeschaltet etc.)                     //
           //-----------------------------------------------------------------//
           lpdis = (LPDRAWITEMSTRUCT) lParam;

           //-----------------------------------------------------------------//
           // weier Hintergrund, schwarzer Rahmen                            //
           //-----------------------------------------------------------------//
           FillRect (lpdis->hDC, &lpdis->rcItem, GetStockObject (GRAY_BRUSH));
           FrameRect(lpdis->hDC, &lpdis->rcItem, GetStockObject (BLACK_BRUSH));

           //-----------------------------------------------------------------//
           // Dreiecke zeichnen                                               //
           //-----------------------------------------------------------------//
           cx = lpdis->rcItem.right - lpdis->rcItem.left;
           cy = lpdis->rcItem.bottom - lpdis->rcItem.top;

           //-----------------------------------------------------------------//
           // Feststellen welches Element neu zu zeichnen ist...              //
           //-----------------------------------------------------------------//
           switch (lpdis->CtlID)
           {
              //--------------------------------------------------------------//
              // "Kleiner" zeichnen                                           //
              //--------------------------------------------------------------//
              case IDC_SMALLER:
                   pt[0].x = 3 * cx / 8; pt[0].y = 1 * cy / 8;
                   pt[1].x = 5 * cx / 8; pt[1].y = 1 * cy / 8;
                   pt[2].x = 4 * cx / 8; pt[2].y = 3 * cy / 8;
                   Triangle(lpdis->hDC, pt);

                   pt[0].x = 7 * cx / 8; pt[0].y = 3 * cy / 8;
                   pt[1].x = 7 * cx / 8; pt[1].y = 5 * cy / 8;
                   pt[2].x = 5 * cx / 8; pt[2].y = 4 * cy / 8;
                   Triangle(lpdis->hDC, pt);

                   pt[0].x = 5 * cx / 8; pt[0].y = 7 * cy / 8;
                   pt[1].x = 3 * cx / 8; pt[1].y = 7 * cy / 8;
                   pt[2].x = 4 * cx / 8; pt[2].y = 5 * cy / 8;
                   Triangle(lpdis->hDC, pt);

                   pt[0].x = 1 * cx / 8; pt[0].y = 5 * cy / 8;
                   pt[1].x = 1 * cx / 8; pt[1].y = 3 * cy / 8;
                   pt[2].x = 3 * cx / 8; pt[2].y = 4 * cy / 8;
                   Triangle(lpdis->hDC, pt);
                   break;

              //--------------------------------------------------------------//
              // "Grer"  zeichnen                                           //
              //--------------------------------------------------------------//
              case IDC_LARGER:
                   pt[0].x = 5 * cx / 8; pt[0].y = 3 * cy / 8;
                   pt[1].x = 3 * cx / 8; pt[1].y = 3 * cy / 8;
                   pt[2].x = 4 * cx / 8; pt[2].y = 1 * cy / 8;
                   Triangle(lpdis->hDC, pt);

                   pt[0].x = 5 * cx / 8; pt[0].y = 5 * cy / 8;
                   pt[1].x = 5 * cx / 8; pt[1].y = 3 * cy / 8;
                   pt[2].x = 7 * cx / 8; pt[2].y = 4 * cy / 8;
                   Triangle(lpdis->hDC, pt);

                   pt[0].x = 3 * cx / 8; pt[0].y = 5 * cy / 8;
                   pt[1].x = 5 * cx / 8; pt[1].y = 5 * cy / 8;
                   pt[2].x = 4 * cx / 8; pt[2].y = 7 * cy / 8;
                   Triangle(lpdis->hDC, pt);

                   pt[0].x = 3 * cx / 8; pt[0].y = 3 * cy / 8;
                   pt[1].x = 3 * cx / 8; pt[1].y = 5 * cy / 8;
                   pt[2].x = 1 * cx / 8; pt[2].y = 4 * cy / 8;
                   Triangle(lpdis->hDC, pt);
                   break;
           }

           //-----------------------------------------------------------------//
           // Eigenes Element invertieren, falls Maustaste gedrckt gehalten  //
           //-----------------------------------------------------------------//
           if (lpdis->itemState & ODS_SELECTED)
              InvertRect(lpdis->hDC, &lpdis->rcItem);

           //-----------------------------------------------------------------//
           // markieren, wenn Element den Eingabefokus hat                    //
           //-----------------------------------------------------------------//
           if (lpdis->itemState & ODS_FOCUS)
           {
           	  lpdis->rcItem.left   += cx / 16;
              lpdis->rcItem.top    += cy / 16;
              lpdis->rcItem.right  -= cx / 16;
              lpdis->rcItem.bottom -= cy / 16;
              DrawFocusRect (lpdis->hDC, &lpdis->rcItem);
           }
           return (0);

      //----------------------------------------------------------------------//
      // Programm beenden                                                     //
      //----------------------------------------------------------------------//
      case WM_DESTROY :
           PostQuitMessage (0);
           return (0);
   }
   return DefWindowProc (hwnd, message, wParam, lParam);
}
