#include <mem.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include "clprint.h"

//------------------------------------------------------------------------------
// Constructor                                                                 -
// Erzeugt einen Port auf den Standardprinter, setzt CPI-Wert auf 12 und den   -
// Zeilenabstand auf 8 LPI. Tabulator wird auf 3 Zeichen voreingestellt        -
// Standardfarbe ist Schwarz                                                   -
//------------------------------------------------------------------------------
TextPrint::TextPrint (DWORD cpi, int tab, COLORREF aColor)
{
	//---------------------------------------------------------------------------
	// Member-Variablen initialisieren                                          -
	//---------------------------------------------------------------------------
	dwCPI         = cpi;        // Festlegung Characters Per Inch               -
   row           = 0;          // aktuelle Zeilennummer                        -
   column        = 0;          // aktuelle Spaltennummer                       -
   page          = 0;          // aktuelle Seitennummer                        -
   iTabWidth     = tab;        // Tabulatorbreite                              -
   isAlias       = FALSE;      // Ist es Kopie eines anderen TextPrint-Objekts?-
   color         = aColor;     // Farbeinstellung                              -
   dwCharExtent  = 0;          //                                              -
   bLineWrapping = TRUE;       // automatischer Zeilenumbruch                  -
   documentON    = FALSE;      //                                              -
   buffsize      = 4098;       // Standard-buffersize fr Strings              -

   headerON      = FALSE;      // Headerdruck eingeschaltet ?                  -
   headerDate    = TRUE;       // Datum im Header drucken ?                    -
   headerPage    = FALSE;      // Seitennummer im Header drucken ?             -
   headerText    = NULL;       // Zeiger auf den Headertext                    -

	//---------------------------------------------------------------------------
	// Printer-Dialog-Struktur initialisieren                                   -
	//---------------------------------------------------------------------------
   memset (&pd, 0, sizeof (PRINTDLG));
   pd.lStructSize = sizeof (PRINTDLG);

	//---------------------------------------------------------------------------
	// Font-Dialog-Struktur initialisieren                                      -
	//---------------------------------------------------------------------------
   memset (&lf, 0, sizeof (LOGFONT));
   strcpy (lf.lfFaceName, STDFONT);   // Standardfont                          -
   lf.lfHeight = STD_HEIGHTUNIT;      // Hhe in Units                         -
   lf.lfWidth  = STD_WIDTHUNIT;       // durchschn. Breite in Units            -

   InitDefaultPrinter ();
}

//------------------------------------------------------------------------------
// Constructor (2) - ALIAS                                                     -
// Erzeugt einen Port auf den Standardprinter, setzt CPI-Wert auf 12 und den   -
// Zeilenabstand auf 8 LPI. Tabulator wird auf 3 Zeichen voreingestellt        -
// Standardfarbe ist Schwarz                                                   -
//------------------------------------------------------------------------------
TextPrint::TextPrint (HDC dc, DWORD cpi, int tab, COLORREF aColor)
{
	//---------------------------------------------------------------------------
	// Member-Variablen initialisieren                                          -
	//---------------------------------------------------------------------------
	dwCPI         = cpi;        // Festlegung Characters Per Inch               -
   row           = 0;          // aktuelle Zeilennummer                        -
   column        = 0;          // aktuelle Spaltennummer                       -
   page          = 0;          // aktuelle Seitennummer                        -
   iTabWidth     = tab;        // Tabulatorbreite                              -
   isAlias       = TRUE;       // Ist es Kopie eines anderen TextPrint-Objekts?-
   color         = aColor;     // Farbeinstellung                              -
   dwCharExtent  = 0;          //                                              -
   bLineWrapping = TRUE;       // automatischer Zeilenumbruch                  -
   documentON    = FALSE;      //                                              -
   buffsize      = 4098;       // Standard-buffersize fr Strings              -

   headerON      = FALSE;      // Headerdruck eingeschaltet ?                  -
   headerDate    = TRUE;       // Datum im Header drucken ?                    -
   headerPage    = FALSE;      // Seitennummer im Header drucken ?             -
   headerText    = NULL;       // Zeiger auf den Headertext                    -

	//---------------------------------------------------------------------------
	// Printer-Dialog-Struktur initialisieren                                   -
	//---------------------------------------------------------------------------
   memset (&pd, 0, sizeof (PRINTDLG));
   pd.lStructSize = sizeof (PRINTDLG);
   pd.hDC = dc;                         // Kopie des Devicekontextes           -

	//---------------------------------------------------------------------------
	// Font-Dialog-Struktur initialisieren                                      -
	//---------------------------------------------------------------------------
   memset (&lf, 0, sizeof (LOGFONT));
   strcpy (lf.lfFaceName, STDFONT);     // Standardfont                        -
   lf.lfHeight = STD_HEIGHTUNIT;        // Hhe in Units                       -
   lf.lfWidth  = STD_WIDTHUNIT;         // durchschn. Breite in Units          -
}

//------------------------------------------------------------------------------
// Destructor                                                                  -
//------------------------------------------------------------------------------
TextPrint::~TextPrint ()
{
   if (headerText) delete [] headerText;    // Headertext freigeben            -
	ClosePrinter ();
}

//------------------------------------------------------------------------------
// Headertext setzen                                                           -
//------------------------------------------------------------------------------
void TextPrint::SetHeaderText(char *x)
{
   if (headerText) delete [] headerText;    // alten Headertext freigeben      -
   headerText = new char [strlen(x)+1];     // neuen Headertext allozieren     -
   if (headerText) strcpy (headerText, x);  // neuen Headertext kopieren       -
}

//------------------------------------------------------------------------------
// Den Standard-Druckeinstellungs-Dialog ffnen und nderungen erlauben        -
//------------------------------------------------------------------------------
BOOL TextPrint::Setup (HWND hwndParent)
{
	BOOL rc = FALSE;
   HDC hdcPrevPrinter;

   //---------------------------------------------------------------------------
	// aktuellen Printer-Device-Context sichern                                 -
   //---------------------------------------------------------------------------
   hdcPrevPrinter = pd.hDC;

   pd.Flags     = PD_RETURNDC | PD_PRINTSETUP | PD_NOSELECTION;
   pd.hwndOwner = hwndParent ? hwndParent : GetActiveWindow ();

   if ((PrintDlg (&pd)) && (pd.hDC))
   {
	   //------------------------------------------------------------------------
      // Bevor die neuen Einstellungen gltig werden knnen, mu das aktuelle  -
      // Dokument abgeschlossen werden                                         -
   	//------------------------------------------------------------------------
      if (documentON)
      {
      	HDC hdcNewPrinter = pd.hDC;
         pd.hDC = hdcPrevPrinter;
         EndDocument ();
         pd.hDC = hdcNewPrinter;
         StartDocument ();
      }
      rc = TRUE;
   }
   return (rc);
}

//------------------------------------------------------------------------------
// Drucker und Dokument initialisieren                                         -
//------------------------------------------------------------------------------
void TextPrint::InitDefaultPrinter (void)
{
	if (!isAlias)
   {
   	//------------------------------------------------------------------------
		// Einen Device-Context auf den System-Drucker holen und ber das Flag   -
      // PD_RETURNDEFAULT sicherstellen, da kein weiterer Dialog offen ist    -
   	//------------------------------------------------------------------------
		pd.Flags = PD_RETURNDC | PD_RETURNDEFAULT | PD_NOSELECTION;
      PrintDlg (&pd);
   }
}

//------------------------------------------------------------------------------
// Drucker freigeben                                                           -
//------------------------------------------------------------------------------
void TextPrint::ClosePrinter (void)
{
	if (pd.hDC)
   {
   	if (documentON)   EndDocument ();
      if (pd.hDevMode)  GlobalFree  (pd.hDevMode);
      if (pd.hDevNames) GlobalFree  (pd.hDevNames);
      if (!isAlias)     DeleteDC    (pd.hDC);
   }
}

//------------------------------------------------------------------------------
// Dokument beenden                                                            -
//------------------------------------------------------------------------------
void TextPrint::EndDocument (void)
{
	EndPage (pd.hDC);
   EndDoc  (pd.hDC);
   documentON = FALSE;
}

//------------------------------------------------------------------------------
// Dokument beginnen                                                           -
//------------------------------------------------------------------------------
void TextPrint::StartDocument (void)
{
	int zw1 = 0;
   int zw2 = 0;
   DOCINFO di = { sizeof (DOCINFO), "TextPrint", NULL };

	if (pd.hDC)
   {
		//------------------------------------------------------------------------
		// Wenn noch ein Dokument offen ist, dann zuerst schlieen               -
		//------------------------------------------------------------------------
		if (documentON) EndDocument ();

		//------------------------------------------------------------------------
		// Seitengre (in Zeichen) neu berechnen                                -
		//------------------------------------------------------------------------
	   zw1 = GetDeviceCaps(pd.hDC, HORZRES) / GetDeviceCaps(pd.hDC, LOGPIXELSX)*
            LOWORD (dwCPI);
   	zw2 = GetDeviceCaps(pd.hDC, VERTRES) / GetDeviceCaps(pd.hDC, LOGPIXELSY)*
            HIWORD (dwCPI);
   	dwCharExtent = MAKELONG (zw1, zw2);
   }
	//---------------------------------------------------------------------------
   // Ein neues Dokument ffnen und auf der ersten Seite beginnen              -
	//---------------------------------------------------------------------------
	StartDoc  (pd.hDC, &di);
   StartPage (pd.hDC);
   row    = 0;
   column = 0;
   page   = 0;
   PageHeader (page+1);

   documentON = TRUE;
}

//------------------------------------------------------------------------------
// Seitenheader drucken                                                        -
//------------------------------------------------------------------------------
#pragma argsused
void TextPrint::PageHeader (int pageNo)
{
	time_t t;
   struct tm *lt;
   char szd [12] = "";
   char szp [12] = "";
   int  zw;             // max. Anzahl druckbarer Zeichen pro Zeile            -
   int  hw;             // Druckbreite fr den Headertext                      -

   hw = GetDeviceCaps(pd.hDC,HORZRES)/GetDeviceCaps(pd.hDC,LOGPIXELSX)*
         LOWORD(dwCPI);

	if (headerON)
   {
		if (headerDate)
      {
      	time (&t);
         lt = localtime (&t);
         sprintf (szd, "%02d.%02d.%04d", lt->tm_mday,lt->tm_mon+1,lt->tm_year);
         hw = hw - (strlen (szd) + 3);
      }

      if (headerPage)
      {
      	sprintf (szp, "%4d", page);
         hw = hw - (strlen (szp) + 3);
      }

      zw = hw - strlen (headerText);
		if (zw <= 0)
         pprintf ("%s%*s%s", szd, hw, headerText, szp);
      else
      {
      	zw = (int)zw/2;
         pprintf ("%s%*s%s%*s%s", szd, zw, "", headerText, zw, "", szp);
      }
   }
}

//------------------------------------------------------------------------------
// Neue Zeichengre setzen, ACHTUNG!!! beendet das aktuelle Dokument          -
//------------------------------------------------------------------------------
void TextPrint::SetCPI (DWORD cpi)
{
	if (documentON) EndDocument ();
   dwCPI = cpi;
}

//------------------------------------------------------------------------------
// Neuen Zeichensatz auswhlen, nur FIX-FONTS erlaubt                          -
//------------------------------------------------------------------------------
BOOL TextPrint::SelectFont (HWND hwndParent)
{
	CHOOSEFONT cf;
   BOOL       rc;

   memset (&cf, 0, sizeof (CHOOSEFONT));
   cf.lStructSize = sizeof (CHOOSEFONT);
   cf.hwndOwner   = hwndParent ? hwndParent : GetActiveWindow ();
   cf.lpLogFont   = &lf;
   cf.Flags       = CF_SCREENFONTS | CF_FORCEFONTEXIST | CF_FIXEDPITCHONLY |
                    CF_INITTOLOGFONTSTRUCT | CF_EFFECTS;
   cf.nFontType   = SCREEN_FONTTYPE;

   rc = ChooseFont (&cf);
   if (rc) color = cf.rgbColors;

   return (rc);
}

//------------------------------------------------------------------------------
// Seite beenden und neue Seite beginnen                                       -
//------------------------------------------------------------------------------
int TextPrint::NextPage (void)
{
	EndPage   (pd.hDC);
   StartPage (pd.hDC);
   column = 0;
   row    = 0;
   page++;
   PageHeader (page+1);
   return (page);
}

//------------------------------------------------------------------------------
// Zeile beenden und neue Zeile beginnen, ggf.auf neue Seite gehen             -
//------------------------------------------------------------------------------
int TextPrint::NextLine (void)
{
   row++;
   column = 0;
   if (HIWORD (dwCharExtent) == row) NextPage ();
   return (row);
}

//------------------------------------------------------------------------------
// Neuen Font im Printer-Device-Context setzen. Da der DC auf die druckbaren   -
// Zeilen und Spalten skaliert ist, wird hier immer 1x1 als Fontgre gesetzt  -
// Die Funktion gibt einen Handle auf den vorhergehenden Font zurck           -
//------------------------------------------------------------------------------
HFONT TextPrint::SetFont (void)
{
	//---------------------------------------------------------------------------
	// Font erzeugen                                                            -
	//---------------------------------------------------------------------------
	lf.lfWidth  = 1;
   lf.lfHeight = 1;
   return ((HFONT)SelectObject (pd.hDC, CreateFontIndirect (&lf)));
}

//------------------------------------------------------------------------------
// Ein einzelnes Zeichen auf dem Drucker ausgeben                              -
// ggf. werden alle notwendigen Aktionen (linefeed, formfeed, CR, tab etc.     -
// durchgefhrt).                                                              -
//------------------------------------------------------------------------------
void TextPrint::WriteChar (char ch)
{
	HFONT hOldFont;

	if (!isOk ()) return;
   if (!documentON) StartDocument ();
   switch (ch)
   {
   	case TP_LF : NextLine ();
      		    	 break;
      case TP_FF : NextPage ();
      			 	 break;
      case TP_CR : column = 0;
                   break;
      case TP_TAB: column += iTabWidth - (column % iTabWidth);
                   if (bLineWrapping && (column >= LOWORD (dwCharExtent)))
                      NextLine ();
                   break;
      default    : if (isprint (ch))
      				 {
                   	 //--------------------------------------------------------
							 // Neuen Font erzeugen und alten ber Handle sichern     -
                   	 //--------------------------------------------------------
                      hOldFont = SetFont ();
                      PrepareDC ();
                      TextOut (pd.hDC, column++, row, &ch, 1);

                   	 //--------------------------------------------------------
							 // Neuen Font wieder vernichten und alten wiederherstellen
                   	 //--------------------------------------------------------
                      DeleteObject (SelectObject (pd.hDC, hOldFont));

                   	 //--------------------------------------------------------
							 // Line-Wrapping, falls notwendig                        -
                   	 //--------------------------------------------------------
							 if (bLineWrapping && column >= LOWORD (dwCharExtent))
                   		 NextLine ();
                   }
                   break;
   } // Switch-Ende
}

//------------------------------------------------------------------------------
// Printer-Device-Context auf Ausgabe vorbereiten                              -
//------------------------------------------------------------------------------
void TextPrint::PrepareDC (void)
{
	int lpxlX = GetDeviceCaps (pd.hDC, LOGPIXELSX);
	int lpxlY = GetDeviceCaps (pd.hDC, LOGPIXELSY);

   SetMapMode (pd.hDC, MM_ANISOTROPIC);
   SetViewportExtEx (pd.hDC, lpxlX, lpxlY, NULL);
   SetWindowExtEx   (pd.hDC, LOWORD(dwCPI), HIWORD(dwCPI), NULL);
   SetViewportOrgEx (pd.hDC, 0, 0, NULL);

	SetBkMode    (pd.hDC, TRANSPARENT);
   SetTextColor (pd.hDC, color);
}

//------------------------------------------------------------------------------
// Methode ist zwecks Verbesserung (Syntaxhighlighting / Parsing) virtuell     -
//------------------------------------------------------------------------------
void TextPrint::WriteString (const char *sz)
{
	for (const char *p = sz; *p; p++) WriteChar (*p);
}

//------------------------------------------------------------------------------
// formatierte Ausgabe wie printf...                                           -
//------------------------------------------------------------------------------
int _cdecl TextPrint::pprintf (const char* fmt, ...)
{
	static const int _bufsize = buffsize;
   va_list argptr;
   int cnt;

   char *buffer = new char [_bufsize];
   if (buffer)
   {
   	va_start (argptr, fmt);
   	cnt = vsprintf (buffer, fmt, argptr);
   	va_end (argptr);

   	WriteString (buffer);

   	delete [] buffer;
   }
   else WriteString ("CLASS TEXTPRINT: ALLOCATION ERROR IN PPRINTF");

   return (cnt);
}

//------------------------------------------------------------------------------
// Puffergre fr formatierte Ausgabe wie printf...                           -
//------------------------------------------------------------------------------
int TextPrint::BufferSize (int x)
{
	if (-1 != x)
	{
		if (x < 4098) buffsize = 4098;
      else buffsize = x;
   }
	return (buffsize);
}


TextPrint& operator<< (TextPrint& tp, char c)
{
	tp.WriteChar (c);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, const char *sz)
{
	tp.WriteString(sz);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, int i)
{
	tp.pprintf ("%d", i);
   return (tp);
}
TextPrint& operator<< (TextPrint& tp, unsigned int ui)
{
	tp.pprintf("%u", ui);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, short s)
{
	tp.pprintf("%hd", s);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, unsigned short us)
{
	tp.pprintf("%hu", us);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, long l)
{
	tp.pprintf("%ld", l);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, unsigned long ul)
{
	tp.pprintf("%lu", ul);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, float f)
{
	tp.pprintf("%f", f);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, double lf)
{
	tp.pprintf("%lf", lf);
   return (tp);
}

TextPrint& operator<< (TextPrint& tp, long double Lf)
{
	tp.pprintf("%Lf", Lf);
   return (tp);
}

