#ifndef _CLPRINT_H_
   #define _CLPRINT_H_

   #include <windows.h>
   #include <commdlg.h>
   #define strict

   #define TP_LF  '\n'
   #define TP_FF  '\f'
   #define TP_CR  '\r'
   #define TP_TAB '\t'

   #define STDFONT "Courier"
   #define STD_HEIGHTUNIT 1
   #define STD_WIDTHUNIT 1

   //---------------------------------------------------------------------------
   // Klassendeklaration                                                       -
   //---------------------------------------------------------------------------
   class TextPrint
   {
      //------------------------------------------------------------------------
      // public Methoden                                                       -
      //------------------------------------------------------------------------
      public:
         //---------------------------------------------------------------------
         // Constructoren / Destructor                                         -
         // erster Constructor ist wg. Vorbelegung aller Parameter zugleich    -
         // der Standardconstructor                                            -
         //---------------------------------------------------------------------
         TextPrint (DWORD cpi=MAKELONG (12,8), int tab=3, COLORREF aCol=0);
         TextPrint (HDC dc,DWORD cpi=MAKELONG(12,8),int tab=3,COLORREF aCol=0);
         virtual ~TextPrint ();

         //---------------------------------------------------------------------
         // Auswahl-Methoden                                                   -
         //---------------------------------------------------------------------
         BOOL Setup           (HWND hwndParent = NULL);
         BOOL SelectFont      (HWND hwndParent = NULL);
         void SetCPI          (DWORD cpi);

         void SetColor        (COLORREF aColor) {color = aColor;}
         void SetLineWrapping (BOOL bSet)       {bLineWrapping = bSet;}
         BOOL Header          (BOOL bSet)       {return (headerON = bSet);}
         void SetHeaderText   (char *x);
         int  BufferSize      (int x = -1);

         //---------------------------------------------------------------------
         // Ausgabe-Methoden                                                   -
         //---------------------------------------------------------------------
         virtual void        WriteString (const char* sz);
                 void        WriteChar   (char ch);
                 int _cdecl  pprintf     (const char *format, ...);

         BOOL  isOk           (void) const {return (NULL != pd.hDC);}
         DWORD GetPageCharExt (void) const {return (dwCharExtent);}

         //---------------------------------------------------------------------
         // Operatoren                                                         -
         //---------------------------------------------------------------------
         operator BOOL () const {return (isOk ());}

      //------------------------------------------------------------------------
      // protected Methoden                                                    -
      //------------------------------------------------------------------------
      protected:
         void  InitDefaultPrinter (void);
         void  ClosePrinter       (void);
         void  PrepareDC          (void);
         void  StartDocument      (void);
         void  EndDocument        (void);
         int   NextLine           (void);
         int   NextPage           (void);
         HFONT SetFont            (void);

         TextPrint (const TextPrint& tp) {}
         TextPrint& operator=  (const TextPrint& tp) {return (*this);}

         virtual void PageHeader (int pageNo);

      //------------------------------------------------------------------------
      // private Methoden (zum Verhindern von Aufrufen automatisch generierten -
      // Codes)                                                                -
      //------------------------------------------------------------------------
		private:
         //---------------------------------------------------------------------
         // Constructoren (Copy-Constructor)                                   -
         //---------------------------------------------------------------------
         // TextPrint (void) {} // schon durch den ersten Constructor mit Vor- -
                                // belegung aller Parameter definiert          -
         TextPrint (TextPrint& x) {}
         bool operator== (const TextPrint& tp) {return false;}

      //------------------------------------------------------------------------
      // Variablen                                                             -
      // Da die Variablen ohnehin nicht direkt zugreifbar sind, sind sie       -
      // weniger interessant als das funktionale Klassen-Interface und stehen  -
      // am Ende der Klasse                                                    -
      //------------------------------------------------------------------------
      protected:
         PRINTDLG pd;             // Struktur fr Standard-Printer-Dialog      -
         LOGFONT  lf;             // Struktur fr Standard-Font-Dialog         -
         int      row;            // aktuelle Zeilennummer                     -
         int      column;         // aktuelle Spaltennummer                    -
         int      page;           // aktuelle Seitennummer                     -
         BOOL     isAlias;        //                                           -
         int      iTabWidth;      // Breite fr Tabulatordarstellung           -
         DWORD    dwCPI;          // Variable fr Zeichenbreite (CPI)          -
         DWORD    dwCharExtent;   //                                           -
         BOOL     documentON;     //                                           -
         int      buffsize;       // Buffergre fr format. Ausgabe (4098+)   -

         BOOL     headerON;       // J/N Header anzeigen                       -
         BOOL     headerDate;     // J/N Datum im Header anzeigen              -
         BOOL     headerPage;     // J/N Seitennummer im Header anzeigen       -
         char    *headerText;     // Pointer auf den Headertext (dynamisch)    -

         COLORREF color;          // RGB-Referenz fr Textfarbe                -
         BOOL     bLineWrapping;  // J/N automatischer Zeilenumbruch           -
   };

   //---------------------------------------------------------------------------
   // berladen der Stream-Operatoren fr bequemeres drucken                   -
   //---------------------------------------------------------------------------
   TextPrint& operator<< (TextPrint& tp, char c);
   TextPrint& operator<< (TextPrint& tp, const char *sz);
   TextPrint& operator<< (TextPrint& tp, int i);
   TextPrint& operator<< (TextPrint& tp, unsigned int ui);
   TextPrint& operator<< (TextPrint& tp, short s);
   TextPrint& operator<< (TextPrint& tp, unsigned short us);
   TextPrint& operator<< (TextPrint& tp, long l);
   TextPrint& operator<< (TextPrint& tp, unsigned long ul);
   TextPrint& operator<< (TextPrint& tp, float f);
   TextPrint& operator<< (TextPrint& tp, double lf);
   TextPrint& operator<< (TextPrint& tp, long double Lf);

#endif

